@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
-/* builtin.c: builtin commands */
-/* (C) 2007-2010 magicant */
+/* builtin.c: built-in commands */
+/* (C) 2007-2011 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -59,17 +59,17 @@
 #endif
 
 
-/* Rules about builtin commands:
- * - `argc' passed to a builtin is at least one; The command name is given in
+/* Rules about built-in commands:
+ * - `argc' passed to a built-in is at least one; The command name is given in
  *   `argv[0]'.
  * - `argv' may be rearranged and the values of the argument strings may be
- *   changed in the builtin. However, the argument strings may not be `free'd or
- *   `realloc'ed.
- * - Builtins may sleep or wait, but cannot be stopped. */
+ *   changed in the built-in. However, the argument strings may not be `free'd
+ *   or `realloc'ed.
+ * - Built-ins may sleep or wait, but cannot be stopped. */
 
 
-/* A hashtable from names of builtins (const char *) to builtin info structures
- * (const builtin_T *). */
+/* A hashtable from names of built-ins (const char *) to built-in info
+ * structures (const builtin_T *). */
 static hashtable_T builtins;
 
 /* Initializes `builtins' */
@@ -194,7 +194,7 @@ void init_builtin(void)
 #undef DEFBUILTIN
 }
 
-/* Returns the builtin command of the specified name or NULL if not found. */
+/* Returns the built-in command of the specified name or NULL if not found. */
 const builtin_T *get_builtin(const char *name)
 {
     return ht_get(&builtins, name).value;
@@ -202,24 +202,23 @@ const builtin_T *get_builtin(const char *name)
 
 #if YASH_ENABLE_HELP
 
-/* Prints description of the specified builtin to stdout.
- * Returns Exit_SUCCESS if the builtin was found and the help was printed.
- * Returns Exit_FAILURE if the builtin was not found or an error occurred. */
+/* Prints description of the specified built-in to the standard output.
+ * Returns Exit_SUCCESS if the built-in was found and the help was printed.
+ * Returns Exit_FAILURE if the built-in was not found or an error occurred. */
 int print_builtin_help(const wchar_t *name)
 {
     char *mbsname = malloc_wcstombs(name);
     const builtin_T *bi = get_builtin(mbsname);
     free(mbsname);
-    if (bi) {
-	if (fputs(gt(bi->help), stdout) == EOF) {
-	    xerror(errno, Ngt("cannot print help of `%ls'"), name);
-	    return Exit_FAILURE;
-	}
-	return Exit_SUCCESS;
-    } else {
-	xerror(0, Ngt("%ls: no such builtin"), name);
+    if (bi == NULL) {
+	xerror(0, Ngt("no such built-in `%ls'"), name);
 	return Exit_FAILURE;
     }
+
+    for (const char **help = bi->help; *help != NULL; help++)
+	if (!xprintf("%s", gt(*help)))
+	    return Exit_FAILURE;
+    return Exit_SUCCESS;
 }
 
 #endif /* YASH_ENABLE_HELP */
@@ -243,20 +242,16 @@ void generate_builtin_candidates(const le_compopt_T *compopt)
     size_t i = 0;
     kvpair_T kv;
     while ((kv = ht_next(&builtins, &i)).key != NULL) {
+	le_candgentype_T type;
 	switch (((const builtin_T *) kv.value)->type) {
-	    case BI_SPECIAL:
-		if (!(compopt->type & CGT_SBUILTIN))
-		    continue;
-		break;
-	    case BI_SEMISPECIAL:
-		if (!(compopt->type & CGT_SSBUILTIN))
-		    continue;
-		break;
-	    case BI_REGULAR:
-		if (!(compopt->type & CGT_RBUILTIN))
-		    continue;
-		break;
+	    case BI_SPECIAL:      type = CGT_SBUILTIN;   break;
+	    case BI_SEMISPECIAL:  type = CGT_SSBUILTIN;  break;
+	    case BI_REGULAR:      type = CGT_RBUILTIN;   break;
+	    default:          assert(false);
 	}
+	if (!(compopt->type & type))
+	    continue;
+
 	if (le_match_comppatterns(compopt, kv.key))
 	    le_new_candidate(CT_COMMAND,
 		    malloc_mbstowcs(kv.key), NULL, compopt);
@@ -266,14 +261,14 @@ void generate_builtin_candidates(const le_compopt_T *compopt)
 #endif /* YASH_ENABLE_LINEEDIT */
 
 
-/* The ":"/"true" builtin */
+/* The ":"/"true" built-in. */
 int true_builtin(
 	int argc __attribute__((unused)), void **argv __attribute__((unused)))
 {
     return EXIT_SUCCESS;
 }
 
-/* The "false" builtin */
+/* The "false" built-in. */
 int false_builtin(
 	int argc __attribute__((unused)), void **argv __attribute__((unused)))
 {
@@ -282,44 +277,50 @@ int false_builtin(
 
 #if YASH_ENABLE_HELP
 
-const char colon_help[] = Ngt(
-": - null utility\n"
+const char *colon_help[] = { Ngt(
+"colon - null utility\n"
+), Ngt(
 "\t: [arg...]\n"
-"Does nothing. Any arguments are ignored.\n"
+), Ngt(
+"The colon built-in does nothing. Any arguments are ignored.\n"
+), Ngt(
 "Note that arguments are expanded and redirections are performed as usual.\n"
-"This command has the same effect as the \"true\" command, but \":\" is a\n"
-"special builtin while \"true\" is a semi-special.\n"
-);
+"The colon and true built-ins have the same effect, but colon is a special\n"
+"built-in while true is a semi-special.\n"
+), NULL };
 
-const char true_help[] = Ngt(
+const char *true_help[] = { Ngt(
 "true - return true value\n"
+), Ngt(
 "\ttrue\n"
-"Does nothing successfully.\n"
+), Ngt(
+"The true built-in does nothing successfully.\n"
 "Any arguments are ignored and the exit status is always zero.\n"
-"This command has the same effect as the \":\" command, but \":\" is a\n"
-"special builtin while \"true\" is a semi-special.\n"
-"Naturally, the opposite of this command is the \"false\" command.\n"
-);
+), Ngt(
+"Note that arguments are expanded and redirections are performed as usual.\n"
+"The colon and true built-ins have the same effect, but colon is a special\n"
+"built-in while true is a semi-special.\n"
+), NULL };
 
-const char false_help[] = Ngt(
+const char *false_help[] = { Ngt(
 "false - return false value\n"
+), Ngt(
 "\tfalse\n"
-"Does nothing unsuccessfully.\n"
+), Ngt(
+"The false built-in does nothing unsuccessfully.\n"
 "Any arguments are ignored and the exit status is always non-zero.\n"
-"Naturally, the opposite of this command is the \"true\" command.\n"
-);
+), NULL };
 
 
-/* The "help" builtin. */
+/* The "help" built-in. */
 int help_builtin(int argc, void **argv)
 {
-    wchar_t opt;
-
-    xoptind = 0, xopterr = true;
-    while ((opt = xgetopt_long(argv, L"", help_option, NULL))) {
-	switch (opt) {
-	    case L'-':  print_help:
-		return print_builtin_help(ARGV(0));
+    const struct xgetopt_T *opt;
+    xoptind = 0;
+    while ((opt = xgetopt(argv, help_option, 0)) != NULL) {
+	switch (opt->shortopt) {
+	    case L'-':
+		goto print_help;
 	    default:
 		fprintf(stderr, gt("Usage:  help command...\n"));
 		return Exit_ERROR;
@@ -327,18 +328,21 @@ int help_builtin(int argc, void **argv)
     }
 
     if (xoptind == argc)
-	goto print_help;
+print_help:
+	return print_builtin_help(ARGV(0));
 
     while (xoptind < argc)
 	print_builtin_help(ARGV(xoptind++));
     return (yash_error_message_count == 0) ? Exit_SUCCESS : Exit_FAILURE;
 }
 
-const char help_help[] = Ngt(
-"help - print usage of builtin\n"
+const char *help_help[] = { Ngt(
+"help - print usage of built-ins\n"
+), Ngt(
 "\thelp command...\n"
-"Prints a description of <command>s.\n"
-);
+), Ngt(
+"The help built-in prints a description of the specified built-in <command>s.\n"
+), NULL };
 
 #endif /* YASH_ENABLE_HELP */
 
