@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* exec.c: command execution */
-/* (C) 2007-2010 magicant */
+/* (C) 2007-2011 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -384,7 +384,7 @@ void exec_for(const command_T *c, bool finally_exit)
 	/* no "in" keyword in the for command: use the positional parameters */
 	struct get_variable v = get_variable(L"@");
 	assert(v.type == GV_ARRAY && v.values != NULL);
-	words = duparray(v.values, copyaswcs);
+	words = pldup(v.values, copyaswcs);
 	count = (int) v.count;
     }
 
@@ -578,7 +578,7 @@ void next_pipe(pipeinfo_T *pi, bool next)
 
 fail:
     pi->pi_tonextfds[PIDX_IN] = pi->pi_tonextfds[PIDX_OUT] = -1;
-    xerror(errno, Ngt("cannot open pipe"));
+    xerror(errno, Ngt("cannot open a pipe"));
 }
 
 /* Connects the pipe(s) and closes the pipes left. */
@@ -726,7 +726,7 @@ pid_t exec_process(
 	    goto done;
 	}
 	if (is_interrupted()) {
-	    recfree(argv, free);
+	    plfree(argv, free);
 	    goto done;
 	}
 	if (argc == 0) {
@@ -844,7 +844,7 @@ done3:
 	close_current_environment();
 done2:
     if (c->c_type == CT_SIMPLE)
-	recfree(argv, free), free(argv0);
+	plfree(argv, free), free(argv0);
 done1:
     undo_redirections(savefd);
 done:
@@ -888,7 +888,7 @@ pid_t fork_and_reset(pid_t pgid, bool fg, sigtype_T sigtype)
     if (cpid != 0) {
 	if (cpid < 0) {
 	    /* fork failure */
-	    xerror(errno, Ngt("fork: cannot make child process"));
+	    xerror(errno, Ngt("cannot make a child process"));
 	} else {
 	    /* parent process */
 	    if (doing_job_control_now && pgid >= 0)
@@ -1113,7 +1113,7 @@ void exec_simple_command(
     switch (ci->type) {
     case externalprogram:
 	if (ci->ci_path == NULL) {
-	    xerror(0, Ngt("%s: no such command or function"), argv0);
+	    xerror(0, Ngt("no such command `%s'"), argv0);
 	    laststatus = Exit_NOTFOUND;
 	} else {
 	    assert(finally_exit);
@@ -1134,9 +1134,9 @@ void exec_simple_command(
 		if (errno_ == EACCES && is_directory(ci->ci_path))
 		    errno_ = EISDIR;
 		if (strcmp(mbsargv[0], ci->ci_path) == 0)
-		    xerror(errno_, Ngt("cannot execute `%s'"), argv0);
+		    xerror(errno_, Ngt("cannot execute command `%s'"), argv0);
 		else
-		    xerror(errno_, Ngt("cannot execute `%s' (%s)"),
+		    xerror(errno_, Ngt("cannot execute command `%s' (%s)"),
 			    argv0, ci->ci_path);
 	    } else if (errno_ != ENOENT) {
 		exec_fall_back_on_sh(argc, mbsargv, environ, ci->ci_path);
@@ -1252,7 +1252,8 @@ void exec_fall_back_on_sh(
 	xexecve(shpath, args, envp);
     else
 	errno = ENOENT;
-    xerror(errno, Ngt("cannot invoke new shell to execute `%s'"), argv[0]);
+    xerror(errno, Ngt("cannot invoke a new shell to execute script `%s'"),
+	    argv[0]);
 }
 
 /* Executes the specified command as a function.
@@ -1307,7 +1308,7 @@ wchar_t *exec_command_substitution(const embedcmd_T *cmdsub)
 
     /* open a pipe to receive output from the command */
     if (pipe(pipefd) < 0) {
-	xerror(errno, Ngt("cannot open pipe for command substitution"));
+	xerror(errno, Ngt("cannot open a pipe for the command substitution"));
 	return NULL;
     }
 
@@ -1328,7 +1329,8 @@ wchar_t *exec_command_substitution(const embedcmd_T *cmdsub)
 	xclose(pipefd[PIDX_OUT]);
 	f = fdopen(pipefd[PIDX_IN], "r");
 	if (!f) {
-	    xerror(errno, Ngt("cannot open pipe for command substitution"));
+	    xerror(errno,
+		    Ngt("cannot open a pipe for the command substitution"));
 	    xclose(pipefd[PIDX_IN]);
 	    lastcmdsubstatus = Exit_NOEXEC;
 	    return NULL;
@@ -1394,7 +1396,7 @@ int exec_iteration(void *const *commands, const char *codename)
 	exec_wcs(*c, codename, false);
 	commandstatus = laststatus;
 	laststatus = savelaststatus;
-	if (iterinfo.exception == ie_break)
+	if (iterinfo.exception == ie_break || is_interrupted())
 	    break;
     }
     iterinfo = saveiterinfo;
@@ -1423,7 +1425,7 @@ int exec_variable_as_commands(const wchar_t *varname, const char *codename)
 	    break;
 	case GV_ARRAY:
 	    /* copy the array values in case they are unset during execution */
-	    array = duparrayn(gv.values, gv.count, copyaswcs);
+	    array = plndup(gv.values, gv.count, copyaswcs);
 	    break;
 	case GV_ARRAY_CONCAT:
 	    /* should execute the concatenated value, but is not supported now*/
@@ -1433,7 +1435,7 @@ int exec_variable_as_commands(const wchar_t *varname, const char *codename)
     }
 
     int result = exec_iteration(array, codename);
-    recfree(array, free);
+    plfree(array, free);
     return result;
 }
 
@@ -1546,29 +1548,44 @@ static bool print_command_absolute_path(
 	const char *name, const char *path, bool humanfriendly)
     __attribute__((nonnull));
 
-/* Options for the "break", "continue" and "eval" builtins. */
-static const struct xoption iter_options[] = {
-    { L"iteration", OPTARG_NONE, L'i', },
+/* Options for the "break", "continue" and "eval" built-ins. */
+static const struct xgetopt_T iter_options[] = {
+    { L'i', L"iteration", OPTARG_NONE, false, NULL, },
 #if YASH_ENABLE_HELP
-    { L"help",      OPTARG_NONE, L'-', },
+    { L'-', L"help",      OPTARG_NONE, false, NULL, },
 #endif
-    { NULL, 0, 0, },
+    { L'\0', NULL, 0, false, NULL, },
 };
 
-/* The "return" builtin */
+/* The "return" built-in. */
 int return_builtin(int argc, void **argv)
 {
-    wchar_t opt;
+    static const struct xgetopt_T options[] = {
+	{ L'n', L"no-return", OPTARG_NONE, false, NULL, },
+#if YASH_ENABLE_HELP
+	{ L'-', L"help",      OPTARG_NONE, false, NULL, },
+#endif
+	{ L'\0', NULL, 0, false, NULL, },
+    };
 
-    xoptind = 0, xopterr = true;
-    while ((opt = xgetopt_long(argv, L"", help_option, NULL))) {
-	switch (opt) {
+    bool noreturn = false;
+
+    const struct xgetopt_T *opt;
+    xoptind = 0;
+    while ((opt = xgetopt(argv, options, 0)) != NULL) {
+	switch (opt->shortopt) {
+	    case L'n':
+		noreturn = true;
+		break;
 #if YASH_ENABLE_HELP
 	    case L'-':
 		return print_builtin_help(ARGV(0));
 #endif
 	    default:  print_usage:
-		fprintf(stderr, gt("Usage:  %ls [n]\n"), ARGV(0));
+		fprintf(stderr,
+			gt(posixly_correct ? Ngt("Usage:  %ls [n]\n")
+			                   : Ngt("Usage:  %ls [-n] [n]\n")),
+			ARGV(0));
 		SPECIAL_BI_ERROR;
 		return Exit_ERROR;
 	}
@@ -1588,30 +1605,38 @@ int return_builtin(int argc, void **argv)
     } else {
 	status = (savelaststatus >= 0) ? savelaststatus : laststatus;
     }
-    execinfo.exception = ee_return;
+    if (!noreturn)
+	execinfo.exception = ee_return;
     return status;
 }
 
 #if YASH_ENABLE_HELP
-const char return_help[] = Ngt(
-"return - return from function\n"
-"\treturn [n]\n"
-"Exits the currently executing function or script file with the exit status\n"
-"of <n>. If <n> is not specified, it defaults to the exit status of the last\n"
-"executed command. <n> should be between 0 and 255 inclusive.\n"
-);
+const char *return_help[] = { Ngt(
+"return - return from a function or script\n"
+), Ngt(
+"\treturn [-n] [n]\n"
+), Ngt(
+"The return built-in exits the currently executing function or script file\n"
+"with the exit status of <n>. If <n> is not specified, it defaults to the\n"
+"exit status of the last executed command. <n> should be between 0 and 255\n"
+"inclusive.\n"
+), Ngt(
+"If the -n (--no-return) option is specified, this built-in does not return\n"
+"from a function or script; simply returns the specified exit status.\n"
+"The -n option cannot be used in the POSIXly correct mode.\n"
+), NULL };
 #endif
 
-/* The "break"/"continue" builtin, which accepts the following option:
+/* The "break"/"continue" built-in, which accepts the following option:
  *  -i: iterative execution */
 int break_builtin(int argc, void **argv)
 {
-    wchar_t opt;
     bool iter = false;
 
-    xoptind = 0, xopterr = true;
-    while ((opt = xgetopt_long(argv, L"i", iter_options, NULL))) {
-	switch (opt) {
+    const struct xgetopt_T *opt;
+    xoptind = 0;
+    while ((opt = xgetopt(argv, iter_options, 0)) != NULL) {
+	switch (opt->shortopt) {
 	    case L'i':
 		iter = true;
 		break;
@@ -1633,7 +1658,7 @@ int break_builtin(int argc, void **argv)
     if (iter) {
 	/* break/continue iteration */
 	if (!iterinfo.iterating) {
-	    xerror(0, Ngt("not in iteration"));
+	    xerror(0, Ngt("not in a iteration"));
 	    return Exit_ERROR;
 	}
 	if (wcscmp(ARGV(0), L"break") == 0) {
@@ -1656,7 +1681,7 @@ int break_builtin(int argc, void **argv)
 		SPECIAL_BI_ERROR;
 		return Exit_ERROR;
 	    } else if (countl == 0) {
-		xerror(0, Ngt("%u: not a positive integer"), 0u);
+		xerror(0, Ngt("%u is not a positive integer"), 0u);
 		SPECIAL_BI_ERROR;
 		return Exit_ERROR;
 	    } else if (countl > UINT_MAX) {
@@ -1667,7 +1692,7 @@ int break_builtin(int argc, void **argv)
 	}
 	assert(count > 0);
 	if (execinfo.loopnest == 0) {
-	    xerror(0, Ngt("not in loop"));
+	    xerror(0, Ngt("not in a loop"));
 	    SPECIAL_BI_ERROR;
 	    return Exit_ERROR;
 	}
@@ -1686,39 +1711,45 @@ int break_builtin(int argc, void **argv)
 
 #if YASH_ENABLE_HELP
 
-const char break_help[] = Ngt(
-"break - exit loop\n"
+const char *break_help[] = { Ngt(
+"break - exit a loop\n"
+), Ngt(
 "\tbreak [n]\n"
 "\tbreak -i\n"
+), Ngt(
 "The first form exits the currently executing for/while/until loop.\n"
 "If <n> is specified, exits the <n>th outer loop.\n"
+), Ngt(
 "The second form, with the -i (--iteration) option, exits the current\n"
 "iterative execution.\n"
-);
+), NULL };
 
-const char continue_help[] = Ngt(
-"continue - continue loop\n"
+const char *continue_help[] = { Ngt(
+"continue - continue a loop\n"
+), Ngt(
 "\tcontinue [n]\n"
 "\tcontinue -i\n"
+), Ngt(
 "The first form ends the current iteration of a for/while/until loop and\n"
 "resumes the next iteration. If <n> is specified, ends the iteration of the\n"
 "<n>th outer loop.\n"
+), Ngt(
 "The second form, with the -i (--iteration) option, ends the execution of\n"
 "commands and resumes the next iteration of the current iterative execution.\n"
-);
+), NULL };
 
 #endif /* YASH_ENABLE_HELP */
 
-/* The "eval" builtin, which accepts the following option:
+/* The "eval" built-in, which accepts the following option:
  *  -i: iterative execution */
 int eval_builtin(int argc __attribute__((unused)), void **argv)
 {
     bool iter = false;
-    wchar_t opt;
-    xoptind = 0, xopterr = true;
-    while ((opt = xgetopt_long(argv,
-		    posixly_correct ? L"+" : L"+i", iter_options, NULL))) {
-	switch (opt) {
+
+    const struct xgetopt_T *opt;
+    xoptind = 0;
+    while ((opt = xgetopt(argv, iter_options, XGETOPT_POSIX)) != NULL) {
+	switch (opt->shortopt) {
 	    case L'i':
 		iter = true;
 		break;
@@ -1746,40 +1777,40 @@ int eval_builtin(int argc __attribute__((unused)), void **argv)
 }
 
 #if YASH_ENABLE_HELP
-const char eval_help[] = Ngt(
-"eval - evaluate arguments as command\n"
+const char *eval_help[] = { Ngt(
+"eval - evaluate arguments as a command\n"
+), Ngt(
 "\teval [-i] [arg...]\n"
-"Parses and executes the specified <arg>s as commands in the current shell\n"
-"environment. Without the -i (--iteration) option, all the <arg>s are joined\n"
-"with a space inserted between each <arg> and the whole resultant string is\n"
-"parsed at a time. With the -i option, <arg>s are parsed and executed one by\n"
-"one (iterative execution).\n"
-"In POSIXly correct mode, the -i option cannot be used.\n"
-);
+), Ngt(
+"The eval built-in parses and executes the specified <arg>s as commands in\n"
+"the current shell environment. Without the -i (--iteration) option, all the\n"
+"<arg>s are joined with a space inserted between each <arg> and the whole\n"
+"resultant string is parsed at a time. With the -i option, <arg>s are parsed\n"
+"and executed one by one (iterative execution).\n"
+"In the POSIXly correct mode, the -i option cannot be used.\n"
+), NULL };
 #endif
 
-/* The "." builtin, which accepts the following option:
+/* The "." built-in, which accepts the following option:
  *  -A: disable aliases
  *  -L: autoload */
 int dot_builtin(int argc, void **argv)
 {
-    static const struct xoption long_options[] = {
-	{ L"no-alias", OPTARG_NONE, L'A', },
-	{ L"autoload", OPTARG_NONE, L'L', },
+    static const struct xgetopt_T options[] = {
+	{ L'A', L"no-alias", OPTARG_NONE, false, NULL, },
+	{ L'L', L"autoload", OPTARG_NONE, false, NULL, },
 #if YASH_ENABLE_HELP
-	{ L"help",     OPTARG_NONE, L'-', },
+	{ L'-', L"help",     OPTARG_NONE, false, NULL, },
 #endif
-	{ NULL, 0, 0, },
+	{ L'\0', NULL, 0, false, NULL, },
     };
 
     bool enable_alias = true, autoload = false;
 
-    wchar_t opt;
-    xoptind = 0, xopterr = true;
-    while ((opt = xgetopt_long(argv,
-		    posixly_correct ? L"+" : L"+AL",
-		    long_options, NULL))) {
-	switch (opt) {
+    const struct xgetopt_T *opt;
+    xoptind = 0;
+    while ((opt = xgetopt(argv, options, XGETOPT_POSIX)) != NULL) {
+	switch (opt->shortopt) {
 	    case L'A':
 		enable_alias = false;
 		break;
@@ -1818,7 +1849,8 @@ int dot_builtin(int argc, void **argv)
 	path = which(mbsfilename, get_path_array(PA_LOADPATH),
 		is_readable_regular);
 	if (path == NULL) {
-	    xerror(0, Ngt("%s: not found in $YASH_LOADPATH"), mbsfilename);
+	    xerror(0, Ngt("file `%s' was not found in $YASH_LOADPATH"),
+		    mbsfilename);
 	    free(mbsfilename);
 	    if (!is_interactive)
 		exit_shell_with_status(Exit_FAILURE);
@@ -1830,7 +1862,7 @@ int dot_builtin(int argc, void **argv)
 	    if (!posixly_correct) {
 		path = mbsfilename;
 	    } else {
-		xerror(0, Ngt("%s: not found in $PATH"), mbsfilename);
+		xerror(0, Ngt("file `%s' was not found in $PATH"), mbsfilename);
 		free(mbsfilename);
 		if (!is_interactive)
 		    exit_shell_with_status(Exit_FAILURE);
@@ -1850,7 +1882,7 @@ int dot_builtin(int argc, void **argv)
     if (path != mbsfilename)
 	free(path);
     if (fd < 0) {
-	xerror(errno, Ngt("cannot open `%s'"), mbsfilename);
+	xerror(errno, Ngt("cannot open file `%s'"), mbsfilename);
 	free(mbsfilename);
 	if (!is_interactive)
 	    exit_shell_with_status(Exit_FAILURE);
@@ -1870,50 +1902,53 @@ int dot_builtin(int argc, void **argv)
 }
 
 #if YASH_ENABLE_HELP
-const char dot_help[] = Ngt(
-"dot - read file and execute commands\n"
+const char *dot_help[] = { Ngt(
+"dot - read a file and execute commands\n"
+), Ngt(
 "\t. [-AL] file [arg...]\n"
-"Reads the specified <file> and executes commands in it.\n"
+), Ngt(
+"The dot built-in reads the specified <file> and executes commands in it.\n"
 "If <arg>s are specified, they are used as the positional parameters.\n"
 "Otherwise, the positional parameters are not changed.\n"
 "If <file> does not contain any slashes, the shell searches $PATH for a\n"
 "readable shell script file whose name is <file>. To ensure that the file in\n"
-"the current working directory is used, start <file> with \"./\".\n"
+"the current working directory is used, start <file> with `./'.\n"
+), Ngt(
 "If the -L (--autoload) option is specified, the shell searches\n"
 "$YASH_LOADPATH instead of $PATH, regardless of whether <file> contains\n"
 "slashes.\n"
+), Ngt(
 "If the -A (--no-alias) option is specified, alias substitution is not\n"
-"performed during processing the file.\n"
-"In POSIXly correct mode, options cannot be used and <arg>s must not be\n"
+"performed while processing the file.\n"
+), Ngt(
+"In the POSIXly correct mode, options cannot be used and <arg>s must not be\n"
 "given.\n"
-);
+), NULL };
 #endif
 
-/* The "exec" builtin, which accepts the following options:
+/* The "exec" built-in, which accepts the following options:
  *  -a name: give <name> as argv[0] to the command
  *  -c: don't pass environment variables to the command
  *  -f: suppress error when we have stopped jobs */
 int exec_builtin(int argc, void **argv)
 {
-    static const struct xoption long_options[] = {
-	{ L"as",    OPTARG_REQUIRED, L'a', },
-	{ L"clear", OPTARG_NONE,     L'c', },
-	{ L"force", OPTARG_NONE,     L'f', },
+    static const struct xgetopt_T options[] = {
+	{ L'a', L"as",    OPTARG_REQUIRED, false, NULL, },
+	{ L'c', L"clear", OPTARG_NONE,     false, NULL, },
+	{ L'f', L"force", OPTARG_NONE,     false, NULL, },
 #if YASH_ENABLE_HELP
-	{ L"help",  OPTARG_NONE,     L'-', },
+	{ L'-', L"help",  OPTARG_NONE,     false, NULL, },
 #endif
-	{ NULL, 0, 0, },
+	{ L'\0', NULL, 0, false, NULL, },
     };
 
-    wchar_t opt;
     const wchar_t *as = NULL;
     bool clear = false, force = false;
 
-    xoptind = 0, xopterr = true;
-    while ((opt = xgetopt_long(argv,
-		    posixly_correct ? L"+" : L"+a:cf",
-		    long_options, NULL))) {
-	switch (opt) {
+    const struct xgetopt_T *opt;
+    xoptind = 0;
+    while ((opt = xgetopt(argv, options, XGETOPT_POSIX)) != NULL) {
+	switch (opt->shortopt) {
 	    case L'a':  as = xoptarg;  break;
 	    case L'c':  clear = true;  break;
 	    case L'f':  force = true;  break;
@@ -1938,12 +1973,11 @@ int exec_builtin(int argc, void **argv)
 	size_t sjc = stopped_job_count();
 	if (sjc > 0) {
 	    fprintf(stderr,
-		    ngt("You have %zu stopped job(s)!",
-			"You have a stopped job!",
+		    ngt("You have a stopped job!",
 			"You have %zu stopped jobs!",
 			sjc),
 		    sjc);
-	    fprintf(stderr, gt("  Use `-f' option to exec anyway.\n"));
+	    fprintf(stderr, gt("  Use the -f option to exec anyway.\n"));
 	    return Exit_FAILURE;
 	}
     }
@@ -1976,7 +2010,7 @@ int exec_builtin_2(int argc, void **argv, const wchar_t *as, bool clear)
     } else {
 	commandpath = get_command_path(args[0], false);
 	if (!commandpath) {
-	    xerror(0, Ngt("%s: no such command"), args[0]);
+	    xerror(0, Ngt("no such command `%s'"), args[0]);
 	    err = Exit_NOTFOUND;
 	    goto err;
 	}
@@ -2015,9 +2049,9 @@ int exec_builtin_2(int argc, void **argv, const wchar_t *as, bool clear)
 	if (errno == EACCES && is_directory(commandpath))
 	    errno = EISDIR;
 	if (strcmp(args[0], commandpath) == 0)
-	    xerror(errno, Ngt("cannot execute `%s'"), args[0]);
+	    xerror(errno, Ngt("cannot execute command `%s'"), args[0]);
 	else
-	    xerror(errno, Ngt("cannot execute `%s' (%s)"),
+	    xerror(errno, Ngt("cannot execute command `%s' (%s)"),
 		    args[0], commandpath);
     } else {
 	exec_fall_back_on_sh(argc, args, envs, commandpath);
@@ -2028,7 +2062,7 @@ int exec_builtin_2(int argc, void **argv, const wchar_t *as, bool clear)
     err = Exit_NOEXEC;
 
     if (envs != environ)
-	recfree((void **) envs, free);
+	plfree((void **) envs, free);
 
 err:
     for (int i = 0; i < argc; i++)
@@ -2040,26 +2074,32 @@ err:
 }
 
 #if YASH_ENABLE_HELP
-const char exec_help[] = Ngt(
-"exec - execute command in the shell process\n"
+const char *exec_help[] = { Ngt(
+"exec - execute a command in the shell process\n"
+), Ngt(
 "\texec [-cf] [-a name] [command [args...]]\n"
-"Replaces the shell process with the specified command. The shell process is\n"
-"`changed' into the new command's process. No child process is created.\n"
-"When an interactive shell has stopped jobs, the -f (--force) option is\n"
-"required to really do exec.\n"
+), Ngt(
+"The exec built-in replaces the shell process with the specified command.\n"
+"The shell process is `changed' into the new command's process. No child\n"
+"process is created. When an interactive shell has stopped jobs, the -f\n"
+"(--force) option is required to perform exec.\n"
+), Ngt(
 "If the -c (--clear) option is specified, the command is executed only with\n"
 "the environment variables assigned for this command.\n"
+), Ngt(
 "If the -a <name> (--as=<name>) option is specified, <name> is passed to the\n"
 "command instead of <command> as the zeroth argument.\n"
+), Ngt(
 "If no <command> is given, the shell does nothing. As a special result,\n"
-"the effects of redirections associated with the \"exec\" command remain\n"
+"the effects of redirections associated with the exec built-in remain\n"
 "after the command.\n"
+), Ngt(
 "In the POSIXly correct mode, none of these options are available and the -f\n"
 "option is always assumed.\n"
-);
+), NULL };
 #endif
 
-/* The "command"/"type" builtin, which accepts the following options:
+/* The "command"/"type" built-in, which accepts the following options:
  *  -a: search aliases
  *  -b: search builtins
  *  -e: search external commands
@@ -2070,19 +2110,19 @@ const char exec_help[] = Ngt(
  *  -V: print info about the command in a human-friendly format */
 int command_builtin(int argc, void **argv)
 {
-    static const struct xoption long_options[] = {
-	{ L"alias",            OPTARG_NONE, L'a', },
-	{ L"builtin-command",  OPTARG_NONE, L'b', },
-	{ L"external-command", OPTARG_NONE, L'e', },
-	{ L"function",         OPTARG_NONE, L'f', },
-	{ L"keyword",          OPTARG_NONE, L'k', },
-	{ L"standard-path",    OPTARG_NONE, L'p', },
-	{ L"identify",         OPTARG_NONE, L'v', },
-	{ L"verbose-identify", OPTARG_NONE, L'V', },
+    static const struct xgetopt_T options[] = {
+	{ L'a', L"alias",            OPTARG_NONE, false, NULL, },
+	{ L'b', L"builtin-command",  OPTARG_NONE, false, NULL, },
+	{ L'e', L"external-command", OPTARG_NONE, false, NULL, },
+	{ L'f', L"function",         OPTARG_NONE, false, NULL, },
+	{ L'k', L"keyword",          OPTARG_NONE, false, NULL, },
+	{ L'p', L"standard-path",    OPTARG_NONE, true,  NULL, },
+	{ L'v', L"identify",         OPTARG_NONE, true,  NULL, },
+	{ L'V', L"verbose-identify", OPTARG_NONE, true,  NULL, },
 #if YASH_ENABLE_HELP
-	{ L"help",             OPTARG_NONE, L'-', },
+	{ L'-', L"help",             OPTARG_NONE, false, NULL, },
 #endif
-	{ NULL, 0, 0, },
+	{ L'\0', NULL, 0, false, NULL, },
     };
 
     bool argv0istype = wcscmp(ARGV(0), L"type") == 0;
@@ -2090,12 +2130,10 @@ int command_builtin(int argc, void **argv)
     enum srchcmdtype_T type = 0;
     bool aliases = false, keywords = false, defpath = false;
 
-    wchar_t opt;
-    xoptind = 0, xopterr = true;
-    while ((opt = xgetopt_long(argv,
-		    posixly_correct ? L"+pvV" : L"+abefkpvV",
-		    long_options, NULL))) {
-	switch (opt) {
+    const struct xgetopt_T *opt;
+    xoptind = 0;
+    while ((opt = xgetopt(argv, options, XGETOPT_POSIX)) != NULL) {
+	switch (opt->shortopt) {
 	    case L'a':  aliases = true;        break;
 	    case L'b':  type |= SCT_BUILTIN;   break;
 	    case L'e':  type |= SCT_EXTERNAL;  break;
@@ -2175,7 +2213,7 @@ int command_builtin_execute(int argc, void **argv, enum srchcmdtype_T type)
     bool finally_exit = false;
 
     if (argv0 == NULL) {
-	xerror(EILSEQ, Ngt("invalid command name"));
+	xerror(EILSEQ, NULL);
 	return Exit_NOTFOUND;
     }
 
@@ -2213,7 +2251,7 @@ bool print_command_info(
     bool found = false;
 
     if (keywords && is_keyword(commandname)) {
-	msgfmt = humanfriendly ? gt("%ls: shell keyword\n") : "%ls\n";
+	msgfmt = humanfriendly ? gt("%ls: a shell keyword\n") : "%ls\n";
 	if (printf(msgfmt, commandname) < 0)
 	    goto ioerror;
 	return true;
@@ -2248,13 +2286,14 @@ bool print_command_info(
 	    }
 	    break;
 	case specialbuiltin:
-	    msgfmt = humanfriendly ? gt("%s: special builtin\n") : "%s\n";
+	    msgfmt = humanfriendly ? gt("%s: a special built-in\n") : "%s\n";
 	    if (printf(msgfmt, name) < 0)
 		goto ioerror;
 	    found = true;
 	    break;
 	case semispecialbuiltin:
-	    msgfmt = humanfriendly ? gt("%s: semi-special builtin\n") : "%s\n";
+	    msgfmt = humanfriendly ? gt("%s: a semi-special built-in\n")
+		                   : "%s\n";
 	    if (printf(msgfmt, name) < 0)
 		goto ioerror;
 	    found = true;
@@ -2267,10 +2306,11 @@ bool print_command_info(
 		cmdpath = get_command_path(name, false);
 	    if (humanfriendly) {
 		if (cmdpath == NULL)
-		    msgfmt = gt("%s: regular builtin (not found in $PATH)\n");
+		    msgfmt =
+			Ngt("%s: a regular built-in (not found in $PATH)\n");
 		else
-		    msgfmt = gt("%s: regular builtin at %s\n");
-		if (printf(msgfmt, name, cmdpath) < 0)
+		    msgfmt = Ngt("%s: a regular built-in at %s\n");
+		if (printf(gt(msgfmt), name, cmdpath) < 0)
 		    goto ioerror;
 	    } else {
 		if (puts(cmdpath == NULL ? name : cmdpath) < 0)
@@ -2279,7 +2319,7 @@ bool print_command_info(
 	    found = true;
 	    break;
 	case function:
-	    msgfmt = humanfriendly ? gt("%s: function\n") : "%s\n";
+	    msgfmt = humanfriendly ? gt("%s: a function\n") : "%s\n";
 	    if (printf(msgfmt, name) < 0)
 		goto ioerror;
 	    found = true;
@@ -2287,12 +2327,12 @@ bool print_command_info(
     }
 
     if (!found && humanfriendly)
-	xerror(0, Ngt("%s: no such command or function"), name);
+	xerror(0, Ngt("no such command `%s'"), name);
     free(name);
     return found;
 
 ioerror:
-    xerror(errno, Ngt("cannot print to standard output"));
+    xerror(errno, Ngt("cannot print to the standard output"));
     free(name);
     return false;
 }
@@ -2305,7 +2345,8 @@ bool print_command_absolute_path(
     if (path[0] == '/') {
 	/* the path is already absolute */
 	if (humanfriendly)
-	    return printf(gt("%s: external command at %s\n"), name, path) >= 0;
+	    return printf(gt("%s: an external command at %s\n"), name, path)
+		>= 0;
 	else
 	    return puts(path) >= 0;
     }
@@ -2327,7 +2368,7 @@ bool print_command_absolute_path(
 
     int r, saveerrno;
     if (humanfriendly)
-	r = printf(gt("%s: external command at %s/%s\n"), name, pwd, path);
+	r = printf(gt("%s: an external command at %s/%s\n"), name, pwd, path);
     else
 	r = printf("%s/%s\n", pwd, path);
     saveerrno = errno;
@@ -2338,23 +2379,29 @@ bool print_command_absolute_path(
 
 #if YASH_ENABLE_HELP
 
-const char command_help[] = Ngt(
-"command - execute or identify command\n"
+const char *command_help[] = { Ngt(
+"command - execute or identify a command\n"
+), Ngt(
 "\tcommand [-befp] command [argument...]\n"
 "\tcommand -v|-V [-abefkp] command...\n"
-"Executes or identifies the specified command.\n"
+), Ngt(
+"The command built-in executes or identifies the specified command.\n"
+), Ngt(
 "Without the -v or -V option, <command> is executed with <argument>s given.\n"
 "<command> is treated as a built-in or external command or a function\n"
 "according to the options specified. If the -p (--standard-path) option is\n"
 "given, the system's default PATH is searched for the command instead of the\n"
 "current $PATH.\n"
+), Ngt(
 "With the -v (--identify) option, <command> is identified. If the command is\n"
 "found in $PATH, its full path is printed. If it is a built-in or a function,\n"
 "the command name is simply printed. If it is an alias, it is printed in the\n"
 "form like \"alias ll='ls -l'\". If the command is not found, nothing is\n"
 "printed and the exit status is non-zero.\n"
+), Ngt(
 "With the -V (--verbose-identify) option, the command is identified in the\n"
 "same way but the result is printed verbosely in a human-readable form.\n"
+), Ngt(
 "The following options specify the command type:\n"
 "  -a --alias\n"
 "  -b --builtin-command\n"
@@ -2363,23 +2410,26 @@ const char command_help[] = Ngt(
 "  -k --keyword\n"
 "When none of these are specified, they default to -be (without -v or -V) or\n"
 "-abefk (with -v or -V)\n"
-);
+), NULL };
 
-const char type_help[] = Ngt(
-"type - identify command\n"
+const char *type_help[] = { Ngt(
+"type - identify a command\n"
+), Ngt(
 "\ttype command...\n"
-"Prints the type of <command>s. Same as \"command -V <command>...\".\n"
-);
+), Ngt(
+"The type built-in prints the type of <command>s.\n"
+"Same as `command -V <command>...'.\n"
+), NULL };
 
 #endif /* YASH_ENABLE_HELP */
 
 /* The "times" builtin */
 int times_builtin(int argc __attribute__((unused)), void **argv)
 {
-    wchar_t opt;
-    xoptind = 0, xopterr = true;
-    while ((opt = xgetopt_long(argv, L"", help_option, NULL))) {
-	switch (opt) {
+    const struct xgetopt_T *opt;
+    xoptind = 0;
+    while ((opt = xgetopt(argv, help_option, 0)) != NULL) {
+	switch (opt->shortopt) {
 #if YASH_ENABLE_HELP
 	    case L'-':
 		return print_builtin_help(ARGV(0));
@@ -2408,7 +2458,7 @@ int times_builtin(int argc __attribute__((unused)), void **argv)
 
     clock = sysconf(_SC_CLK_TCK);
     if (times(&tms) == (clock_t) -1) {
-	xerror(errno, Ngt("cannot get time data"));
+	xerror(errno, Ngt("cannot get the time data"));
 	return Exit_FAILURE;
     }
     format_time(tms.tms_utime, sum, sus);
@@ -2419,22 +2469,24 @@ int times_builtin(int argc __attribute__((unused)), void **argv)
 	    sum, sus, ssm, sss, cum, cus, csm, css) >= 0) {
 	return Exit_SUCCESS;
     } else {
-	xerror(errno, Ngt("cannot print to standard output"));
+	xerror(errno, Ngt("cannot print to the standard output"));
 	return Exit_FAILURE;
     }
 #undef format_time
 }
 
 #if YASH_ENABLE_HELP
-const char times_help[] = Ngt(
+const char *times_help[] = { Ngt(
 "times - print process times\n"
+), Ngt(
 "\ttimes\n"
-"Prints the accumulated user and system times consumed by the shell process\n"
-"and all of its child processes.\n"
+), Ngt(
+"The times built-in prints the accumulated user and system times consumed by\n"
+"the shell process and all of its child processes.\n"
 "The first output line is for the shell process, and the second the child\n"
-"processes. For each line, the user time is printed, followed by the system\n"
+"processes. For each line, the user time is printed followed by the system\n"
 "time.\n"
-);
+), NULL };
 #endif
 
 
