@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* expand.c: word expansion */
-/* (C) 2007-2010 magicant */
+/* (C) 2007-2011 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -143,7 +143,7 @@ bool expand_line(void *const *restrict args,
 
     while (*args) {
 	if (!expand_multiple(*args, &list)) {
-	    recfree(pl_toary(&list), free);
+	    plfree(pl_toary(&list), free);
 	    return false;
 	}
 	args++;
@@ -167,17 +167,17 @@ bool expand_multiple(const wordunit_T *w, plist_T *list)
     if (!expand_word_and_split(w, pl_init(&templist))) {
 	if (!is_interactive)
 	    exit_shell_with_status(Exit_EXPERROR);
-	recfree(pl_toary(&templist), free);
+	plfree(pl_toary(&templist), free);
 	return false;
     }
 
     /* glob */
-    if (shopt_noglob) {
+    if (shopt_glob) {
+	glob_all(pl_toary(&templist), list);
+    } else {
 	for (size_t i = 0; i < templist.length; i++)
 	    pl_add(list, unescapefree(templist.contents[i]));
 	pl_destroy(&templist);
-    } else {
-	glob_all(pl_toary(&templist), list);
     }
 
     return true;
@@ -197,8 +197,8 @@ bool expand_word_and_split(const wordunit_T *restrict w, plist_T *restrict list)
 
     /* four expansions (w -> list1) */
     if (!expand_word(w, tt_single, false, &valuelist1, &splitlist1)) {
-	recfree(pl_toary(&valuelist1), free);
-	recfree(pl_toary(&splitlist1), free);
+	plfree(pl_toary(&valuelist1), free);
+	plfree(pl_toary(&splitlist1), free);
 	return false;
     }
 
@@ -236,7 +236,7 @@ wchar_t *expand_single(const wordunit_T *arg, tildetype_T tilde)
     if (!expand_word(arg, tilde, false, &list, NULL)) {
 	if (!is_interactive)
 	    exit_shell_with_status(Exit_EXPERROR);
-	recfree(pl_toary(&list), free);
+	plfree(pl_toary(&list), free);
 	return NULL;
     }
     if (list.length != 1) {
@@ -244,7 +244,7 @@ wchar_t *expand_single(const wordunit_T *arg, tildetype_T tilde)
 	const wchar_t *ifs = getvar(L VAR_IFS);
 	wchar_t padding[] = { ifs ? ifs[0] : L' ', L'\0' };
 	result = joinwcsarray(list.contents, padding);
-	recfree(pl_toary(&list), free);
+	plfree(pl_toary(&list), free);
     } else {
 	result = list.contents[0];
 	pl_destroy(&list);
@@ -271,12 +271,7 @@ char *expand_single_with_glob(const wordunit_T *arg, tildetype_T tilde)
 	return NULL;
 
     /* glob */
-    if (shopt_noglob || !is_pathname_matching_pattern(exp)) {
-noglob:
-	result = realloc_wcstombs(unescapefree(exp));
-	if (!result)
-	    xerror(EILSEQ, Ngt("redirection"));
-    } else {
+    if (shopt_glob && is_pathname_matching_pattern(exp)) {
 	plist_T list;
 	bool ok;
 
@@ -286,7 +281,7 @@ noglob:
 	set_interruptible_by_sigint(false);
 	if (!ok) {
 	    free(exp);
-	    recfree(pl_toary(&list), free);
+	    plfree(pl_toary(&list), free);
 	    xerror(EINTR, Ngt("redirection"));
 	    result = NULL;
 	} else if (list.length == 1) {
@@ -296,16 +291,22 @@ noglob:
 		xerror(EILSEQ, Ngt("redirection"));
 	    pl_destroy(&list);
 	} else {
-	    recfree(pl_toary(&list), free);
+	    plfree(pl_toary(&list), free);
 	    if (posixly_correct) {
 		goto noglob;
 	    } else {
 		exp = unescapefree(exp);
-		xerror(0, Ngt("%ls: not single file"), exp);
+		xerror(0, Ngt("filename `%ls' matches more than one file"),
+			exp);
 		free(exp);
 		result = NULL;
 	    }
 	}
+    } else {
+noglob:
+	result = realloc_wcstombs(unescapefree(exp));
+	if (!result)
+	    xerror(EILSEQ, Ngt("redirection"));
     }
     return result;
 }
@@ -635,7 +636,7 @@ bool expand_param(const paramexp_T *p, bool indq, struct expand_word_T *e)
 	    startindex = 0, endindex = SSIZE_MAX;
 	    free(start);
 	    if (p->pe_end) {
-		xerror(0, Ngt("invalid parameter index"));
+		xerror(0, Ngt("the parameter index is invalid"));
 		return false;
 	    }
 	} else if (!evaluate_index(start, &startindex)) {
@@ -663,7 +664,7 @@ bool expand_param(const paramexp_T *p, bool indq, struct expand_word_T *e)
 	plist_T plist;
 	pl_init(&plist);
 	if (!expand_word(p->pe_nest, tt_none, true, &plist, NULL)) {
-	    recfree(pl_toary(&plist), free);
+	    plfree(pl_toary(&plist), free);
 	    return false;
 	}
 	v.type = (plist.length == 1) ? GV_SCALAR : GV_ARRAY;
@@ -743,14 +744,14 @@ treat_array:
 #endif
 		assert(0 <= startindex && startindex <= endindex);
 		if (save)
-		    list = duparrayn(v.values + startindex,
+		    list = plndup(v.values + startindex,
 			    endindex - startindex, copyaswcs);
 		else
 		    list = trim_array(v.values, startindex, endindex);
 		break;
 	    case idx_number:
 		if (!save)
-		    recfree(v.values, free);
+		    plfree(v.values, free);
 		list = xmalloc(2 * sizeof *list);
 		list[0] = malloc_wprintf(L"%zu", v.count);
 		list[1] = NULL;
@@ -779,24 +780,26 @@ treat_array:
     case PT_MINUS:
 	if (unset) {
 subst:
-	    recfree(list, free);
+	    plfree(list, free);
 	    return expand_word_inner(p->pe_subst, tt_single, indq, true, e);
 	}
 	break;
     case PT_ASSIGN:
 	if (unset) {
-	    recfree(list, free);
+	    plfree(list, free);
 	    if (p->pe_type & PT_NEST) {
-		xerror(0, Ngt("invalid assignment in parameter expansion"));
+		xerror(0,
+		    Ngt("a nested parameter expansion cannot be assigned"));
 		return false;
 	    } else if (!is_name(p->pe_name)) {
-		xerror(0, Ngt("cannot assign to `%ls' in parameter expansion"),
+		xerror(0, Ngt("cannot assign to parameter `%ls' "
+			    "in parameter expansion"),
 			p->pe_name);
 		return false;
 	    } else if ((v.type == GV_ARRAY_CONCAT)
 		    || (v.type == GV_ARRAY && startindex + 1 != endindex)) {
-                xerror(0, Ngt("cannot assign to an element of array `%ls' "
-                            "in parameter expansion: invalid index"),
+                xerror(0, Ngt("the specified index does not support assignment "
+			    "in the parameter expansion of array `%ls'"),
 			p->pe_name);
 		return false;
 	    }
@@ -826,16 +829,16 @@ subst:
 	break;
     case PT_ERROR:
 	if (unset) {
-	    recfree(list, free);
+	    plfree(list, free);
 	    print_subst_as_error(p);
 	    return false;
 	}
 	break;
     }
 
-    if (shopt_nounset && unset) {
-	recfree(list, free);
-	xerror(0, Ngt("%ls: parameter not set"), p->pe_name);
+    if (unset && !shopt_unset) {
+	plfree(list, free);
+	xerror(0, Ngt("parameter `%ls' is not set"), p->pe_name);
 	return false;
     }
 
@@ -844,7 +847,7 @@ subst:
     case PT_MATCH:
 	match = expand_single(p->pe_match, tt_single);
 	if (!match) {
-	    recfree(list, free);
+	    plfree(list, free);
 	    return false;
 	}
 	match_each(list, match, p->pe_type);
@@ -856,7 +859,7 @@ subst:
 	if (!match || !subst) {
 	    free(match);
 	    free(subst);
-	    recfree(list, free);
+	    plfree(list, free);
 	    return false;
 	}
 	subst = unescapefree(subst);
@@ -871,7 +874,7 @@ subst:
 	const wchar_t *ifs = getvar(L VAR_IFS);
 	wchar_t padding[] = { ifs ? ifs[0] : L' ', L'\0' };
 	wchar_t *chain = joinwcsarray(list, padding);
-	recfree(list, free);
+	plfree(list, free);
 	list = xmalloc(2 * sizeof *list);
 	list[0] = chain;
 	list[1] = NULL;
@@ -953,7 +956,7 @@ wchar_t *expand_param_simple(const paramexp_T *p)
 
     void **results = pl_toary(expand.valuelist);
     wchar_t *result = ok ? joinwcsarray(results, L" ") : NULL;
-    recfree(results, free);
+    plfree(results, free);
     return result;
 }
 
@@ -1054,11 +1057,11 @@ void print_subst_as_error(const paramexp_T *p)
     } else {
 	/* use the default error message */
 	if (p->pe_type & PT_NEST)
-	    xerror(0, Ngt("parameter null"));
+	    xerror(0, Ngt("the parameter value is empty"));
 	else
 	    xerror(0, (p->pe_type & PT_COLON)
-		    ? Ngt("%ls: parameter null or not set")
-		    : Ngt("%ls: parameter not set"),
+		    ? Ngt("parameter `%ls' is not set or has an empty value")
+		    : Ngt("parameter `%ls' is not set"),
 		    p->pe_name);
     }
 }
@@ -1434,7 +1437,7 @@ void fieldsplit(wchar_t *restrict s, char *restrict split,
 /* Performs field splitting.
  * `valuelist' is a NULL-terminated array of pointers to wide strings to split.
  * `splitlist' is an array of pointers to corresponding splittability strings.
- * `valuelist' and `splitlist' are `recfree'ed in this function.
+ * `valuelist' and `splitlist' are `plfree'ed in this function.
  * The results are added to `dest'. */
 void fieldsplit_all(void **restrict valuelist, void **restrict splitlist,
 	plist_T *restrict dest)
@@ -1687,7 +1690,7 @@ wchar_t *escaped_wcspbrk(const wchar_t *s, const wchar_t *accept)
 enum wglbflags get_wglbflags(void)
 {
     enum wglbflags flags = 0;
-    if (shopt_nocaseglob)   flags |= WGLB_CASEFOLD;
+    if (!shopt_caseglob)    flags |= WGLB_CASEFOLD;
     if (shopt_dotglob)      flags |= WGLB_PERIOD;
     if (shopt_markdirs)     flags |= WGLB_MARK;
     if (shopt_extendedglob) flags |= WGLB_RECDIR;
@@ -1696,7 +1699,7 @@ enum wglbflags get_wglbflags(void)
 
 /* Performs file name expansion to the specified patterns.
  * `patterns' is a NULL-terminated array of pointers to `free'able wide strings
- * cast to (void *). `patterns' is `recfree'd in this function.
+ * cast to (void *). `patterns' is `plfree'd in this function.
  * The results are added to `list' as newly-malloced wide strings. */
 void glob_all(void **restrict patterns, plist_T *restrict list)
 {
