@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* history.c: command history management */
-/* (C) 2007-2010 magicant */
+/* (C) 2007-2011 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -1099,7 +1099,7 @@ static int history_write(const wchar_t *s)
     __attribute__((nonnull));
 static void history_refresh_file(void);
 
-/* The "fc" builtin, which accepts the following options:
+/* The "fc" built-in, which accepts the following options:
  *  -e: specify the editor to edit history
  *  -l: list history
  *  -n: don't print entry numbers
@@ -1108,30 +1108,28 @@ static void history_refresh_file(void);
  *  -v: print time for each entry */
 int fc_builtin(int argc, void **argv)
 {
-    static const struct xoption long_options[] = {
-	{ L"editor",     OPTARG_REQUIRED, L'e', },
-	{ L"list",       OPTARG_NONE,     L'l', },
-	{ L"no-numbers", OPTARG_NONE,     L'n', },
-	{ L"quiet",      OPTARG_NONE,     L'q', },
-	{ L"reverse",    OPTARG_NONE,     L'r', },
-	{ L"silent",     OPTARG_NONE,     L's', },
-	{ L"verbose",    OPTARG_NONE,     L'v', },
+    static const struct xgetopt_T options[] = {
+	{ L'e', L"editor",     OPTARG_REQUIRED, true,  NULL, },
+	{ L'l', L"list",       OPTARG_NONE,     true,  NULL, },
+	{ L'n', L"no-numbers", OPTARG_NONE,     true,  NULL, },
+	{ L'q', L"quiet",      OPTARG_NONE,     false, NULL, },
+	{ L'r', L"reverse",    OPTARG_NONE,     true,  NULL, },
+	{ L's', L"silent",     OPTARG_NONE,     true,  NULL, },
+	{ L'v', L"verbose",    OPTARG_NONE,     false, NULL, },
 #if YASH_ENABLE_HELP
-	{ L"help",       OPTARG_NONE,     L'-', },
+	{ L'-', L"help",       OPTARG_NONE,     false, NULL, },
 #endif
-	{ NULL, 0, 0, },
+	{ L'\0', NULL, 0, false, NULL, },
     };
 
     const wchar_t *editor = NULL;
     bool list = false, quiet = false, rev = false, silent = false;
     enum fcprinttype_T ptype = FC_NUMBERED;
 
-    wchar_t opt;
-    xoptind = 0, xopterr = true;
-    while ((opt = xgetopt_long(argv,
-		    posixly_correct ? L"-e:lnrs" : L"-e:lnqrsv",
-		    long_options, NULL))) {
-	switch (opt) {
+    const struct xgetopt_T *opt;
+    xoptind = 0;
+    while ((opt = xgetopt(argv, options, XGETOPT_DIGIT))) {
+	switch (opt->shortopt) {
 	    case L'e':  editor = xoptarg;        break;
 	    case L'l':  list   = true;           break;
 	    case L'n':  ptype  = FC_UNNUMBERED;  break;
@@ -1170,7 +1168,7 @@ int fc_builtin(int argc, void **argv)
 	if (list) {
 	    return Exit_SUCCESS;
 	} else {
-	    xerror(0, Ngt("history is empty"));
+	    xerror(0, Ngt("the command history is empty"));
 	    return Exit_FAILURE;
 	}
     }
@@ -1237,7 +1235,7 @@ int fc_builtin(int argc, void **argv)
 	}
 	if (efirst == Histlist) {
 	    assert(vfirst != NULL);
-	    xerror(0, Ngt("%ls: no such entry"), vfirst);
+	    xerror(0, Ngt("no such history entry `%ls'"), vfirst);
 	    return Exit_FAILURE;
 	}
     }
@@ -1250,7 +1248,7 @@ int fc_builtin(int argc, void **argv)
 	    elast = get_nth_newest_entry(-last);
 	if (elast == Histlist) {
 	    assert(vlast != NULL);
-	    xerror(0, Ngt("%ls: no such entry"), vlast);
+	    xerror(0, Ngt("no such history entry `%ls'"), vlast);
 	    return Exit_FAILURE;
 	}
     }
@@ -1316,7 +1314,7 @@ histentry_T *fc_search_entry(const wchar_t *prefix)
 	e = search_entry(s, false);
     free(s);
     if (e == Histlist)
-	xerror(0, Ngt("no such entry beginning with `%ls'"), prefix);
+	xerror(0, Ngt("no such history entry beginning with `%ls'"), prefix);
     return e;
 }
 
@@ -1352,7 +1350,7 @@ int fc_print_entries(
 		assert(false);
 	}
 	if (r < 0) {
-	    xerror(errno, Ngt("cannot print to standard output"));
+	    xerror(errno, Ngt("cannot print to the standard output"));
 	    return Exit_FAILURE;
 	}
 	if (e == end)
@@ -1424,12 +1422,12 @@ int fc_edit_and_exec_entries(
 
     fd = create_temporary_file(&temp, S_IRUSR | S_IWUSR);
     if (fd < 0) {
-	xerror(errno, Ngt("cannot create temporary file to edit history"));
+	xerror(errno, Ngt("cannot create a temporary file to edit history"));
 	goto error1;
     }
     f = fdopen(fd, "w");
     if (!f) {
-	xerror(errno, Ngt("cannot open temporary file to edit history"));
+	xerror(errno, Ngt("cannot open temporary file `%s'"), temp);
 	xclose(fd);
 	goto error2;
     }
@@ -1437,10 +1435,10 @@ int fc_edit_and_exec_entries(
     savelaststatus = laststatus;
     cpid = fork_and_reset(0, true, 0);
     if (cpid < 0) {  // fork failed
-	xerror(0, Ngt("cannot invoke editor to edit history"));
+	xerror(0, Ngt("cannot invoke the editor to edit history"));
 	fclose(f);
 	if (unlink(temp) < 0)
-	    xerror(errno, Ngt("cannot remove temporary file `%s'"), temp);
+	    xerror(errno, Ngt("failed to remove temporary file `%s'"), temp);
 error2:
 	free(temp);
 error1:
@@ -1459,15 +1457,15 @@ error1:
 
 	int fd;
 	if (laststatus != Exit_SUCCESS) {
-	    xerror(0, Ngt("editor returned non-zero status"));
+	    xerror(0, Ngt("the editor returned a non-zero exit status"));
 	    fd = -1;
 	} else {
 	    fd = move_to_shellfd(open(temp, O_RDONLY));
 	    if (fd < 0)
-		xerror(errno, Ngt("cannot read command from `%s'"), temp);
+		xerror(errno, Ngt("cannot read commands from file `%s'"), temp);
 	}
 	if (unlink(temp) < 0)
-	    xerror(errno, Ngt("cannot remove temporary file `%s'"), temp);
+	    xerror(errno, Ngt("failed to remove temporary file `%s'"), temp);
 	free(temp);
 
 	if (fd < 0)
@@ -1524,29 +1522,41 @@ void fc_read_history(FILE *f, bool quiet)
 }
 
 #if YASH_ENABLE_HELP
-const char fc_help[] = Ngt(
+const char *fc_help[] = { Ngt(
 "fc - list or re-execute command history\n"
+), Ngt(
 "\tfc [-qr] [-e editor] [first [last]]\n"
 "\tfc -s [-q] [old=new] [first]\n"
 "\tfc -l [-nrv] [first [last]]\n"
+), Ngt(
 "The first form invokes an editor to edit a temporary file containing the\n"
 "command history and, after the editor exited, executes commands in the file.\n"
+), Ngt(
 "The second form, with the -s (--silent) option, re-executes commands in the\n"
-"history without invoking an editor. If <old=new> is given, <old> in the\n"
-"command is replaced with <new> before execution.\n"
+"history without invoking an editor. If <old=new> is given, string <old> in\n"
+"the command is replaced with <new> before execution.\n"
+), Ngt(
 "The third form, with the -l (--list) option, prints the command history. In\n"
 "this form commands are not re-executed.\n"
+), (
 "\n"
+), Ngt(
 "The -e (--editor) option can be used to specify the editor. If this option\n"
 "is not given, the value of $FCEDIT is used as the default editor. If $FCEDIT\n"
-"is not set either, \"ed\" is the last resort.\n"
+"is not set either, `ed' is the last resort.\n"
+), Ngt(
 "The -n (--no-numbers) option suppresses command numbers, which would\n"
 "otherwise be printed preceding each command.\n"
+), Ngt(
 "The -q (--quiet) option suppresses echoing the executed command.\n"
+), Ngt(
 "The -r (--reverse) option reverses the order of commands to be edited or\n"
 "printed.\n"
-"The -v (--verbose) option makes the listing more detailed.\n"
+), Ngt(
+"The -v (--verbose) option prints command dates in the listing.\n"
+), (
 "\n"
+), Ngt(
 "The range of command history to be edited or printed can be specified by\n"
 "parameters <first> and <last>. If the value of <first> or <last> is an\n"
 "integer, it is considered a history number. If it is negative, the command\n"
@@ -1555,12 +1565,14 @@ const char fc_help[] = Ngt(
 "command: it indicates the most recent command beginning with it.\n"
 "If <first> is omitted, it defaults to -16 (with -l) or -1 (without -l).\n"
 "If <last> is omitted, it defaults to -1 (with -l) or <first> (without -l).\n"
+), (
 "\n"
-"In POSIXly correct mode, the -q and -v options are not available.\n"
-);
+), Ngt(
+"In the POSIXly correct mode, the -q and -v options are not available.\n"
+), NULL };
 #endif /* YASH_ENABLE_HELP */
 
-/* The "history" builtin, which accepts the following options:
+/* The "history" built-in, which accepts the following options:
  *  -c: clear whole history
  *  -d: remove history entry
  *  -r: read history from a file
@@ -1569,17 +1581,17 @@ const char fc_help[] = Ngt(
  *  -F: flush history file */
 int history_builtin(int argc, void **argv)
 {
-    static const struct xoption long_options[] = {
-	{ L"clear",      OPTARG_NONE,     L'c', },
-	{ L"delete",     OPTARG_REQUIRED, L'd', },
-	{ L"read",       OPTARG_REQUIRED, L'r', },
-	{ L"set",        OPTARG_REQUIRED, L's', },
-	{ L"write",      OPTARG_REQUIRED, L'w', },
-	{ L"flush-file", OPTARG_NONE,     L'F', },
+    static const struct xgetopt_T options[] = {
+	{ L'c', L"clear",      OPTARG_NONE,     true,  NULL, },
+	{ L'd', L"delete",     OPTARG_REQUIRED, true,  NULL, },
+	{ L'r', L"read",       OPTARG_REQUIRED, true,  NULL, },
+	{ L's', L"set",        OPTARG_REQUIRED, true,  NULL, },
+	{ L'w', L"write",      OPTARG_REQUIRED, true,  NULL, },
+	{ L'F', L"flush-file", OPTARG_NONE,     true,  NULL, },
 #if YASH_ENABLE_HELP
-	{ L"help",       OPTARG_NONE,     L'-', },
+	{ L'-', L"help",       OPTARG_NONE,     false, NULL, },
 #endif
-        { NULL, 0, 0, },
+        { L'\0', NULL, 0, false, NULL, },
     };
 
     if (hist_lock) {
@@ -1589,12 +1601,13 @@ int history_builtin(int argc, void **argv)
     maybe_init_history();
 
     bool hasoption = false, removedthis = false;
-    wchar_t opt;
     int result;
-    xoptind = 0, xopterr = true;
-    while ((opt = xgetopt_long(argv, L"cd:r:s:w:F", long_options, NULL))) {
+
+    const struct xgetopt_T *opt;
+    xoptind = 0;
+    while ((opt = xgetopt(argv, options, 0)) != NULL) {
 	hasoption = true;
-	switch (opt) {
+	switch (opt->shortopt) {
 	    case L'c':
 		history_clear_all();
 		break;
@@ -1639,7 +1652,7 @@ int history_builtin(int argc, void **argv)
     /* print history */
     if (xoptind < argc) {
 	if (xoptind + 1 != argc) {
-	    xerror(0, Ngt("too many arguments"));
+	    xerror(0, Ngt("too many operands are specified"));
 	    goto print_usage;
 	}
 
@@ -1713,7 +1726,7 @@ int history_delete(const wchar_t *s)
 		e = get_nth_newest_entry((unsigned) n);
 	}
 	if (e == Histlist)
-	    xerror(0, Ngt("%ls: no such entry"), s);
+	    xerror(0, Ngt("no such history entry `%ls'"), s);
     }
 
     if (e != Histlist) {
@@ -1759,7 +1772,7 @@ int history_read(const wchar_t *s)
     return Exit_SUCCESS;
 
 error:
-    xerror(errno, Ngt("cannot read history from `%ls'"), s);
+    xerror(errno, Ngt("cannot read history from file `%ls'"), s);
     return Exit_FAILURE;
 }
 
@@ -1791,7 +1804,7 @@ int history_write(const wchar_t *s)
     return Exit_SUCCESS;
 
 error:
-    xerror(errno, Ngt("cannot write history to `%ls'"), s);
+    xerror(errno, Ngt("cannot write history to file `%ls'"), s);
     return Exit_FAILURE;
 }
 
@@ -1811,20 +1824,28 @@ void history_refresh_file(void)
 }
 
 #if YASH_ENABLE_HELP
-const char history_help[] = Ngt(
+const char *history_help[] = { Ngt(
 "history - manage command history\n"
+), Ngt(
 "\thistory [-cF] [-d entry] [-s command] [-r file] [-w file] [n]\n"
-"Without options, prints the command history. The number of entries to print\n"
-"can be specified as the argument <n>.\n"
+), Ngt(
+"Without options, the history built-in prints the command history. The number\n"
+"of entries to print can be specified by argument <n>.\n"
+), Ngt(
 "The -c (--clear) option clears the command history completely.\n"
+), Ngt(
 "The -d (--delete) option deletes the specified <entry>. You can specify\n"
 "<entry> by the number or by the prefix.\n"
+), Ngt(
 "The -s (--set) option replaces the last history entry with <command>.\n"
+), Ngt(
 "The -r (--read) option reads history entries from <file>.\n"
+), Ngt(
 "The -w (--write) option writes all the history entries into <file>.\n"
+), Ngt(
 "The -F (--flush-file) option rebuilds the history file, removing unused old\n"
 "data.\n"
-);
+), NULL };
 #endif
 
 
