@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* input.c: functions for input of command line */
-/* (C) 2007-2010 magicant */
+/* (C) 2007-2011 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -208,7 +208,7 @@ inputresult_T input_interactive(struct xwcsbuf_T *buf, void *inputinfo)
 #if YASH_ENABLE_LINEEDIT
     /* read a line using line editing */
     if (info->fileinfo->fd == STDIN_FILENO
-	    && shopt_lineedit != shopt_nolineedit) {
+	    && shopt_lineedit != SHOPT_NOLINEEDIT) {
 	wchar_t *line;
 	inputresult_T result;
 
