@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* lineedit.c: command line editing */
-/* (C) 2007-2010 magicant */
+/* (C) 2007-2011 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -382,11 +382,11 @@ char pop_prebuffer(void)
 bool has_meta_bit(char c)
 {
     switch (shopt_le_convmeta) {
-	case shopt_yes:
+	case SHOPT_YES:
 	    break;
-	case shopt_no:
+	case SHOPT_NO:
 	    return false;
-	case shopt_auto:
+	case SHOPT_AUTO:
 	    if (le_meta_bit8)
 		break;
 	    else
