@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* path.c: filename-related utilities */
-/* (C) 2007-2010 magicant */
+/* (C) 2007-2011 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -1042,9 +1042,9 @@ static inline mode_t copy_group_mask(mode_t mode)
 static inline mode_t copy_other_mask(mode_t mode)
     __attribute__((const));
 
-/* The "cd" builtin, which accepts the following options:
- *  -L: don't resolve symlinks (default)
- *  -P: resolve symlinks
+/* The "cd" built-in, which accepts the following options:
+ *  -L: don't resolve symbolic links (default)
+ *  -P: resolve symbolic links
  *  --default-directory=<dir>: go to <dir> when the operand is missing
  * -L and -P are mutually exclusive: the one specified last is used. */
 int cd_builtin(int argc, void **argv)
@@ -1052,10 +1052,10 @@ int cd_builtin(int argc, void **argv)
     bool logical = true;
     const wchar_t *newpwd = NULL;
 
-    wchar_t opt;
-    xoptind = 0, xopterr = true;
-    while ((opt = xgetopt_long(argv, L"-LP", cd_options, NULL))) {
-	switch (opt) {
+    const struct xgetopt_T *opt;
+    xoptind = 0;
+    while ((opt = xgetopt(argv, cd_options, XGETOPT_DIGIT)) != NULL) {
+	switch (opt->shortopt) {
 	    case L'L':  logical = true;    break;
 	    case L'P':  logical = false;   break;
 	    case L'd':  newpwd = xoptarg;  break;
@@ -1064,7 +1064,7 @@ int cd_builtin(int argc, void **argv)
 		return print_builtin_help(ARGV(0));
 #endif
 	    default:  print_usage:
-		fprintf(stderr, gt("Usage:  %ls [-L|-P] [dir]\n"), L"cd");
+		fprintf(stderr, gt("Usage:  %ls [-L|-P] [directory]\n"), L"cd");
 		return Exit_ERROR;
 	}
     }
@@ -1075,7 +1075,7 @@ int cd_builtin(int argc, void **argv)
 	    if (newpwd == NULL) {
 		newpwd = getvar(L VAR_HOME);
 		if (newpwd == NULL || newpwd[0] == L'\0') {
-		    xerror(0, Ngt("$HOME not set"));
+		    xerror(0, Ngt("$HOME is not set"));
 		    return Exit_FAILURE;
 		}
 	    }
@@ -1084,7 +1084,7 @@ int cd_builtin(int argc, void **argv)
 	    if (wcscmp(ARGV(xoptind), L"-") == 0) {
 		newpwd = getvar(L VAR_OLDPWD);
 		if (newpwd == NULL || newpwd[0] == L'\0') {
-		    xerror(0, Ngt("$OLDPWD not set"));
+		    xerror(0, Ngt("$OLDPWD is not set"));
 		    return Exit_FAILURE;
 		}
 		printnewdir = true;
@@ -1114,7 +1114,7 @@ int change_directory(const wchar_t *newpwd, bool printnewdir, bool logical)
     oldpwd = getvar(L VAR_PWD);
     if (oldpwd == NULL || oldpwd[0] != L'/') {
 	if (oldpwd == newpwd) {
-	    xerror(0, Ngt("invalid $PWD value"));
+	    xerror(0, Ngt("$PWD has an invalid value"));
 	    return Exit_FAILURE;
 	}
 	/* we have to assure `oldpwd != newpwd' because we're going to
@@ -1123,7 +1123,7 @@ int change_directory(const wchar_t *newpwd, bool printnewdir, bool logical)
 	char *pwd = xgetcwd();
 	if (pwd == NULL) {
 	    if (logical) {
-		xerror(errno, Ngt("cannot determine current directory"));
+		xerror(errno, Ngt("cannot determine the current directory"));
 		return Exit_FAILURE;
 	    }
 	} else {
@@ -1302,39 +1302,48 @@ bool starts_with_root_parent(const wchar_t *path)
 }
 
 #if YASH_ENABLE_HELP
-const char cd_help[] = Ngt(
-"cd - change directory\n"
-"\tcd [-L|-P] [dir]\n"
-"Changes the working directory to <dir>.\n"
-"If <dir> is \"-\", the working directory is changed to $OLDPWD.\n"
-"If <dir> is a relative path which does not start with \".\" or \"..\",\n"
+const char *cd_help[] = { Ngt(
+"cd - change the working directory\n"
+), Ngt(
+"\tcd [-L|-P] [directory]\n"
+), Ngt(
+"The cd built-in changes the working directory to <dir>.\n"
+"If <dir> is `-', the working directory is changed to $OLDPWD.\n"
+"If <dir> is a relative path that does not start with `.' or `..',\n"
 "paths in $CDPATH are searched to find a new directory.\n"
 "If the working directory is successfully changed, $PWD is set to the new\n"
 "directory and $OLDPWD is set to the previous $PWD.\n"
+), (
+"\n"
+), Ngt(
 "When the -L (--logical) option is specified, symbolic links in $PWD are left\n"
 "unchanged. When the -P (--physical) option is specified, symbolic links are\n"
 "resolved so that $PWD does not contain any symbolic links.\n"
-"-L and -P are mutually exclusive: the last specified one is used.\n"
+), Ngt(
+"-L and -P are mutually exclusive: only the last specified one is effective.\n"
 "If neither is specified, -L is the default.\n"
+), (
+"\n"
+), Ngt(
 "If <dir> is not specified, the working directory is changed to the directory\n"
 "specified by the --default-directory=... option. If this option is not\n"
 "specified either, the default is $HOME.\n"
-);
+), NULL };
 #endif
 
-/* The "pwd" builtin, which accepts the following options:
- *  -L: don't resolve symlinks (default)
- *  -P: resolve symlinks
+/* The "pwd" built-in, which accepts the following options:
+ *  -L: don't resolve symbolic links (default)
+ *  -P: resolve symbolic links
  * -L and -P are mutually exclusive: the one specified last is used. */
 int pwd_builtin(int argc __attribute__((unused)), void **argv)
 {
     bool logical = true;
     char *mbspwd;
-    wchar_t opt;
 
-    xoptind = 0, xopterr = true;
-    while ((opt = xgetopt_long(argv, L"-LP", pwd_options, NULL))) {
-	switch (opt) {
+    const struct xgetopt_T *opt;
+    xoptind = 0;
+    while ((opt = xgetopt(argv, pwd_options, XGETOPT_DIGIT)) != NULL) {
+	switch (opt->shortopt) {
 	    case L'L':  logical = true;   break;
 	    case L'P':  logical = false;  break;
 #if YASH_ENABLE_HELP
@@ -1364,53 +1373,59 @@ int pwd_builtin(int argc __attribute__((unused)), void **argv)
 
     mbspwd = xgetcwd();
     if (mbspwd == NULL) {
-	xerror(errno, Ngt("cannot determine current directory"));
+	xerror(errno, Ngt("cannot determine the current directory"));
 	return Exit_FAILURE;
     }
 print:
     if (printf("%s\n", mbspwd) < 0)
-	xerror(errno, Ngt("cannot print to standard output"));
+	xerror(errno, Ngt("cannot print to the standard output"));
     free(mbspwd);
     return (yash_error_message_count == 0) ? Exit_SUCCESS : Exit_FAILURE;
 }
 
 #if YASH_ENABLE_HELP
-const char pwd_help[] = Ngt(
-"pwd - print working directory\n"
+const char *pwd_help[] = { Ngt(
+"pwd - print the working directory\n"
+), Ngt(
 "\tpwd [-L|-P]\n"
-"Prints the absolute pathname of the current working directory.\n"
+), Ngt(
+"The pwd built-in prints the absolute pathname of the current working\n"
+"directory.\n"
+), (
+"\n"
+), Ngt(
 "When the -L (--logical) option is specified, $PWD is printed if it is\n"
-"correct. It may contain symbolic links in the pathname.\n"
+"correct. It may contain symbolic links.\n"
 "When the -P (--physical) option is specified, the printed pathname does not\n"
 "contain any symbolic links.\n"
-"-L and -P are mutually exclusive: the last specified one is used.\n"
+), Ngt(
+"-L and -P are mutually exclusive: only the last specified one is effective.\n"
 "If neither is specified, -L is the default.\n"
-);
+), NULL };
 #endif
 
-/* The "hash" builtin, which accepts the following options:
+/* The "hash" built-in, which accepts the following options:
  *  -a: print all entries
  *  -d: use the directory cache
  *  -r: remove cache entries */
 int hash_builtin(int argc, void **argv)
 {
-    static const struct xoption long_options[] = {
-	{ L"all",       OPTARG_NONE, L'a', },
-	{ L"directory", OPTARG_NONE, L'd', },
-	{ L"remove",    OPTARG_NONE, L'r', },
+    static const struct xgetopt_T options[] = {
+	{ L'a', L"all",       OPTARG_NONE, false, NULL, },
+	{ L'd', L"directory", OPTARG_NONE, false, NULL, },
+	{ L'r', L"remove",    OPTARG_NONE, true,  NULL, },
 #if YASH_ENABLE_HELP
-	{ L"help",      OPTARG_NONE, L'-', },
+	{ L'-', L"help",      OPTARG_NONE, false, NULL, },
 #endif
-	{ NULL, 0, 0, },
+	{ L'\0', NULL, 0, false, NULL, },
     };
 
     bool remove = false, all = false, dir = false;
 
-    wchar_t opt;
-    xoptind = 0, xopterr = true;
-    while ((opt = xgetopt_long(argv, posixly_correct ? L"r" : L"adr",
-		    long_options, NULL))) {
-	switch (opt) {
+    const struct xgetopt_T *opt;
+    xoptind = 0;
+    while ((opt = xgetopt(argv, options, 0)) != NULL) {
+	switch (opt->shortopt) {
 	    case L'a':  all    = true;  break;
 	    case L'd':  dir    = true;  break;
 	    case L'r':  remove = true;  break;
@@ -1439,7 +1454,7 @@ int hash_builtin(int argc, void **argv)
 	    } else {                // remember the specified
 		for (int i = xoptind; i < argc; i++)
 		    if (!get_home_directory(ARGV(i), true))
-			xerror(0, Ngt("%ls: no such user"), ARGV(i));
+			xerror(0, Ngt("no such user `%ls'"), ARGV(i));
 	    }
 	}
     } else {
@@ -1461,15 +1476,16 @@ int hash_builtin(int argc, void **argv)
 	    } else {                // remember the specified
 		for (int i = xoptind; i < argc; i++) {
 		    if (wcschr(ARGV(i), L'/')) {
-			xerror(0, Ngt("%ls: command name must not contain `/'"),
-				ARGV(i));
+			xerror(0, Ngt("`%ls': a command name must not contain "
+				    "`/'"), ARGV(i));
 			continue;
 		    }
 
 		    char *cmd = malloc_wcstombs(ARGV(i));
 		    if (cmd) {
 			if (!get_command_path(cmd, true))
-			    xerror(0, Ngt("%s: not found in $PATH"), cmd);
+			    xerror(0, Ngt("command `%s' was not found "
+					"in $PATH"), cmd);
 			free(cmd);
 		    }
 		}
@@ -1497,7 +1513,7 @@ void print_command_paths(bool all)
     while ((kv = ht_next(&cmdhash, &index)).key) {
 	if (all || !get_builtin(kv.key)) {
 	    if (printf("%s\n", (char *) kv.value) < 0) {
-		xerror(errno, Ngt("cannot print to standard output"));
+		xerror(errno, Ngt("cannot print to the standard output"));
 		break;
 	    }
 	}
@@ -1515,52 +1531,58 @@ void print_home_directories(void)
     while ((kv = ht_next(&homedirhash, &index)).key) {
 	const wchar_t *key = kv.key, *value = kv.value;
 	if (printf("~%ls=%ls\n", key, value) < 0) {
-	    xerror(errno, Ngt("cannot print to standard output"));
+	    xerror(errno, Ngt("cannot print to the standard output"));
 	    break;
 	}
     }
 }
 
 #if YASH_ENABLE_HELP
-const char hash_help[] = Ngt(
-"hash - remember, forget or report command locations\n"
+const char *hash_help[] = { Ngt(
+"hash - remember, forget, or report command locations\n"
+), Ngt(
 "\thash command...\n"
 "\thash -r [command...]\n"
 "\thash [-a]\n"
 "\thash -d user...\n"
 "\thash -d -r [user...]\n"
 "\thash -d\n"
+), Ngt(
 "The first form immediately performs command path search and caches\n"
-"<command>s' fullpaths.\n"
+"<command>s' full paths.\n"
+), Ngt(
 "The second form, using the -r (--remove) option, removes the paths of\n"
 "<command>s (or all the paths if none specified) from the cache. Note that\n"
 "assignment to $PATH also removes all paths from the cache.\n"
+), Ngt(
 "The third form prints the currently cached paths. Without the -a (--all)\n"
-"option, paths for builtin commands are not printed.\n"
+"option, paths for built-in commands are not printed.\n"
+), Ngt(
 "With the -d (--directory) option, this command does the same things to the\n"
 "home directory cache, rather than the command path cache.\n"
+), Ngt(
 "In the POSIXly correct mode, the -r option is the only available option.\n"
-);
+), NULL };
 #endif
 
-/* The "umask" builtin, which accepts the following option:
+/* The "umask" built-in, which accepts the following option:
  *  -S: symbolic output */
 int umask_builtin(int argc, void **argv)
 {
-    static const struct xoption long_options[] = {
-	{ L"symbolic", OPTARG_NONE, L'S', },
+    static const struct xgetopt_T options[] = {
+	{ L'S', L"symbolic", OPTARG_NONE, true,  NULL, },
 #if YASH_ENABLE_HELP
-	{ L"help",     OPTARG_NONE, L'-', },
+	{ L'-', L"help",     OPTARG_NONE, false, NULL, },
 #endif
-	{ NULL, 0, 0, },
+	{ L'\0', NULL, 0, false, NULL, },
     };
 
-    wchar_t opt;
     bool symbolic = false;
 
-    xoptind = 0, xopterr = true;
-    while ((opt = xgetopt_long(argv, L"S", long_options, NULL))) {
-	switch (opt) {
+    const struct xgetopt_T *opt;
+    xoptind = 0;
+    while ((opt = xgetopt(argv, options, 0)) != NULL) {
+	switch (opt->shortopt) {
 	    case L'S':
 		symbolic = true;
 		break;
@@ -1596,7 +1618,7 @@ bool print_umask_octal(mode_t mode)
     if (printf("0%.3jo\n", (uintmax_t) mode) >= 0) {
 	return Exit_SUCCESS;
     } else {
-	xerror(errno, Ngt("cannot print to standard output"));
+	xerror(errno, Ngt("cannot print to the standard output"));
 	return Exit_FAILURE;
     }
 }
@@ -1627,7 +1649,7 @@ bool print_umask_symbolic(mode_t mode)
     if (!ferror(stdout)) {
 	return Exit_SUCCESS;
     } else {
-	xerror(errno, Ngt("cannot print to standard output"));
+	xerror(errno, Ngt("cannot print to the standard output"));
 	return Exit_FAILURE;
     }
 }
@@ -1642,7 +1664,7 @@ int set_umask(const wchar_t *maskstr)
 	errno = 0;
 	mask = wcstoumax(maskstr, &end, 8);
 	if (errno || *end) {
-	    xerror(0, Ngt("`%ls': invalid mask"), maskstr);
+	    xerror(0, Ngt("`%ls' is not a valid mask specification"), maskstr);
 	    return Exit_ERROR;
 	}
 	umask((mode_t) (mask & (S_IRWXU | S_IRWXG | S_IRWXO)));
@@ -1724,7 +1746,7 @@ parse_end:
 
 err:
     umask(~origmask);
-    xerror(0, Ngt("`%ls': invalid mask"), savemaskstr);
+    xerror(0, Ngt("`%ls' is not a valid mask specification"), savemaskstr);
     return Exit_ERROR;
 }
 
@@ -1750,15 +1772,17 @@ mode_t copy_other_mask(mode_t mode)
 }
 
 #if YASH_ENABLE_HELP
-const char umask_help[] = Ngt(
-"umask - print or set file creation mask\n"
+const char *umask_help[] = { Ngt(
+"umask - print or set the file creation mask\n"
+), Ngt(
 "\tumask mode\n"
 "\tumask [-S]\n"
-"Sets the file mode creation mask of the shell to <mode>. The mask will be\n"
-"inherited by succeedingly invoked commands.\n"
+), Ngt(
+"The umask built-in sets the file mode creation mask of the shell to <mode>.\n"
+"The mask will be inherited by commands invoked by the shell.\n"
 "If <mode> is not specified, the current setting of the mask is printed.\n"
 "The -S (--symbolic) option makes a symbolic output.\n"
-);
+), NULL };
 #endif
 
 
