@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* redir.c: manages file descriptors and provides functions for redirections */
-/* (C) 2007-2010 magicant */
+/* (C) 2007-2011 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -253,8 +253,8 @@ void open_ttyfd(void)
     if (ttyfd < 0) {
 	ttyfd = move_to_shellfd(open("/dev/tty", O_RDWR));
 	if (ttyfd < 0) {
-	    xerror(errno, Ngt("cannot open `%s'"), "/dev/tty");
-	    xerror(0, Ngt("job control disabled"));
+	    xerror(errno, Ngt("cannot open file `%s'"), "/dev/tty");
+	    xerror(0, Ngt("disabling job control"));
 	    do_job_control = false;
 	}
     }
@@ -302,17 +302,13 @@ bool open_redirections(const redir_T *r, savefd_T **save)
 
     while (r) {
 	if (r->rd_fd < 0 || is_shellfd(r->rd_fd)) {
-	    xerror(0, Ngt("redirection: file descriptor %d unavailable"),
+	    xerror(0, Ngt("redirection: file descriptor %d is unavailable"),
 		    r->rd_fd);
 	    return false;
 	}
 
 	/* expand rd_filename */
-#ifdef NDEBUG
-	char *filename;
-#else
-	char *filename = filename;
-#endif
+	char *INIT(filename);
 	switch (r->rd_type) {
 	    case RT_INPUT:  case RT_OUTPUT:  case RT_CLOBBER:  case RT_APPEND:
 	    case RT_INOUT:  case RT_DUPIN:   case RT_DUPOUT:   case RT_PIPE:
@@ -337,7 +333,7 @@ bool open_redirections(const redir_T *r, savefd_T **save)
 	    flags = O_RDONLY;
 	    goto openwithflags;
 	case RT_OUTPUT:
-	    if (shopt_noclobber && !is_irregular_file(filename)) {
+	    if (!shopt_clobber && !is_irregular_file(filename)) {
 		flags = O_WRONLY | O_CREAT | O_EXCL;
 	    } else {
 	case RT_CLOBBER:
@@ -354,7 +350,8 @@ openwithflags:
 	    keepopen = false;
 	    fd = open_file(filename, flags);
 	    if (fd < 0) {
-		xerror(errno, Ngt("redirection: cannot open `%s'"), filename);
+		xerror(errno, Ngt("redirection: cannot open file `%s'"),
+			filename);
 		free(filename);
 		return false;
 	    }
@@ -513,7 +510,7 @@ int open_socket(const char *hostandport, int socktype)
 	wport = wcschr(whostandport, L'/');
 	if (wport) {
 	    hostname = malloc_wcsntombs(whostandport, wport - whostandport);
-	    port = malloc_wcstombs(wport + 1);
+	    port = malloc_wcstombs(wport + 1); // TODO return value null check
 	} else {
 	    hostname = xstrdup(hostandport);
 	    port = NULL;
@@ -535,7 +532,8 @@ int open_socket(const char *hostandport, int socktype)
     free(hostname);
     free(port);
     if (err != 0) {
-	xerror(0, Ngt("socket redirection: cannot resolve address of %s: %s"),
+	xerror(0, Ngt("socket redirection: "
+		    "cannot resolve the address of `%s': %s"),
 		hostandport, gai_strerror(err));
 	set_interruptible_by_sigint(false);
 	errno = saveerrno;
@@ -583,7 +581,7 @@ int parse_and_check_dup(char *const num, redirtype_T type)
     }
 
     if (is_shellfd(fd)) {
-	xerror(0, Ngt("redirection: file descriptor %d unavailable"),
+	xerror(0, Ngt("redirection: file descriptor %d is unavailable"),
 		fd);
 	fd = -2;
 	goto end;
@@ -593,7 +591,7 @@ int parse_and_check_dup(char *const num, redirtype_T type)
 	/* check the read/write permission */
 	int flags = fcntl(fd, F_GETFL);
 	if (flags < 0) {
-	    xerror(errno, Ngt("redirection: %d"), fd);
+	    xerror(errno, Ngt("redirection: %s"), num);
 	    fd = -2;
 	} else if (type == RT_DUPIN) {
 	    switch (flags & O_ACCMODE) {
@@ -601,7 +599,8 @@ int parse_and_check_dup(char *const num, redirtype_T type)
 		    /* ok */
 		    break;
 		default:
-		    xerror(0, Ngt("redirection: %d: not readable"), fd);
+		    xerror(0, Ngt("redirection: "
+				"file descriptor %d is not readable"), fd);
 		    fd = -2;
 		    break;
 	    }
@@ -612,7 +611,8 @@ int parse_and_check_dup(char *const num, redirtype_T type)
 		    /* ok */
 		    break;
 		default:
-		    xerror(0, Ngt("redirection: %d: not writable"), fd);
+		    xerror(0, Ngt("redirection: "
+				"file descriptor %d is not writable"), fd);
 		    fd = -2;
 		    break;
 	    }
@@ -650,11 +650,11 @@ int parse_and_exec_pipe(int outputfd, char *num, savefd_T **save)
 	fd = -1;
     } else if (outputfd == inputfd) {
 	xerror(0, Ngt("redirection: %d>>|%d: "
-		    "same input and output file descriptors"),
+		    "the input and output file descriptors are same"),
 		outputfd, inputfd);
 	fd = -1;
     } else if (is_shellfd(inputfd)) {
-	xerror(0, Ngt("redirection: file descriptor %d unavailable"),
+	xerror(0, Ngt("redirection: file descriptor %d is unavailable"),
 		inputfd);
 	fd = -1;
     } else {
@@ -708,7 +708,8 @@ int open_heredocument(const wordunit_T *contents)
 
     char *mcontents = realloc_wcstombs(wcontents);
     if (!mcontents) {
-	xerror(EILSEQ, Ngt("cannot write here-document contents"));
+	xerror(EILSEQ, Ngt("cannot write the here-document contents "
+		    "to the temporary file"));
 	return -1;
     }
 
@@ -747,7 +748,8 @@ int open_herestring(char *s, bool appendnewline)
 	    /* It is guaranteed that all the contents is written to the pipe
 	     * at once, so we don't have to use `write_all' here. */
 	    if (write(pipefd[PIDX_OUT], s, len) < 0)
-		xerror(errno, Ngt("cannot write here-document contents"));
+		xerror(errno, Ngt("cannot write the here-document contents "
+			    "to the temporary file"));
 	    xclose(pipefd[PIDX_OUT]);
 	    free(s);
 	    return pipefd[PIDX_IN];
@@ -758,7 +760,8 @@ int open_herestring(char *s, bool appendnewline)
     char *tempfile;
     fd = create_temporary_file(&tempfile, 0);
     if (fd < 0) {
-	xerror(errno, Ngt("cannot create temporary file for here-document"));
+	xerror(errno,
+		Ngt("cannot create a temporary file for the here-document"));
 	free(s);
 	return -1;
     }
@@ -766,10 +769,12 @@ int open_herestring(char *s, bool appendnewline)
 	xerror(errno, Ngt("failed to remove temporary file `%s'"), tempfile);
     free(tempfile);
     if (!write_all(fd, s, len))
-	xerror(errno, Ngt("cannot write here-document contents"));
+	xerror(errno, Ngt("cannot write the here-document contents "
+		    "to the temporary file"));
     free(s);
     if (lseek(fd, 0, SEEK_SET) != 0)
-	xerror(errno, Ngt("cannot seek temporary file for here-document"));
+	xerror(errno,
+		Ngt("cannot seek the temporary file for the here-document"));
     return fd;
 }
 
@@ -783,8 +788,8 @@ int open_process_redirection(const embedcmd_T *command, redirtype_T type)
 
     assert(type == RT_PROCIN || type == RT_PROCOUT);
     if (pipe(pipefd) < 0) {
-	xerror(errno,
-		Ngt("redirection: cannot open pipe for command redirection"));
+	xerror(errno, Ngt("redirection: cannot open a pipe "
+		    "for the command redirection"));
 	return -1;
     }
     cpid = fork_and_reset(-1, false, 0);
