@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* sig.c: signal handling */
-/* (C) 2007-2010 magicant */
+/* (C) 2007-2011 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -747,7 +747,7 @@ void execute_exit_trap(void)
 void set_trap(int signum, const wchar_t *command)
 {
     if (signum == SIGKILL || signum == SIGSTOP) {
-	xerror(0, Ngt("SIG%ls: cannot be trapped"),
+	xerror(0, Ngt("SIG%ls cannot be trapped"),
 		signum == SIGKILL ? L"KILL" : L"STOP");
 	return;
     }
@@ -762,7 +762,7 @@ void set_trap(int signum, const wchar_t *command)
 	    commandp = &rttrap_command[index];
 	    receivedp = &rtsignal_received[index];
 	} else {
-	    xerror(0, Ngt("SIG%ls: unsupported real-time signal"),
+	    xerror(0, Ngt("real-time signal SIG%ls is not supported"),
 		    get_signal_name(signum));
 	    return;
 	}
@@ -778,7 +778,7 @@ void set_trap(int signum, const wchar_t *command)
 	/* Signals that were ignored on entry to a non-interactive shell cannot
 	 * be trapped or reset. (POSIX) */
 #if FIXED_SIGNAL_AS_ERROR
-	xerror(0, Ngt("SIG%ls: cannot be reset"), get_signal_name(signum));
+	xerror(0, Ngt("SIG%ls cannot be reset"), get_signal_name(signum));
 #endif
 	return;
     }
@@ -1020,25 +1020,23 @@ static bool print_signal(int signum, const wchar_t *name, bool verbose)
 static void signal_job(int signum, const wchar_t *jobname)
     __attribute__((nonnull));
 
-/* The "trap" builtin */
+/* The "trap" built-in. */
 int trap_builtin(int argc, void **argv)
 {
-    static const struct xoption long_options[] = {
-	{ L"print", OPTARG_NONE, L'p', },
+    static const struct xgetopt_T options[] = {
+	{ L'p', L"print", OPTARG_NONE, false, NULL, },
 #if YASH_ENABLE_HELP
-	{ L"help",  OPTARG_NONE, L'-', },
+	{ L'-', L"help",  OPTARG_NONE, false, NULL, },
 #endif
-	{ NULL, 0, 0, },
+	{ L'\0', NULL, 0, false, NULL, },
     };
 
-    wchar_t opt;
     bool print = false;
 
-    xoptind = 0, xopterr = true;
-    while ((opt = xgetopt_long(argv,
-		    posixly_correct ? L"" : L"p",
-		    long_options, NULL))) {
-	switch (opt) {
+    const struct xgetopt_T *opt;
+    xoptind = 0;
+    while ((opt = xgetopt(argv, options, 0)) != NULL) {
+	switch (opt->shortopt) {
 	    case L'p':
 		print = true;
 		break;
@@ -1084,7 +1082,7 @@ int trap_builtin(int argc, void **argv)
 	    wchar_t *name = ARGV(xoptind);
 	    int signum = get_signal_number_toupper(name);
 	    if (signum < 0) {
-		xerror(0, Ngt("%ls: no such signal"), name);
+		xerror(0, Ngt("no such signal `%ls'"), name);
 	    } else {
 #if defined SIGRTMIN && defined SIGRTMAX
 		if (sigrtmin <= signum && signum <= sigrtmax) {
@@ -1127,7 +1125,7 @@ set_traps:
 	if (signum >= 0)
 	    set_trap(signum, command);
 	else
-	    xerror(0, Ngt("%ls: no such signal"), name);
+	    xerror(0, Ngt("no such signal `%ls'"), name);
     } while (++xoptind < argc);
     return (yash_error_message_count == 0) ? Exit_SUCCESS : Exit_FAILURE;
 
@@ -1157,7 +1155,7 @@ bool print_trap(const wchar_t *signame, const wchar_t *command)
 	wchar_t *q = quote_sq(command);
 	int r = printf("trap -- %ls %ls\n", q, signame);
 	if (r < 0)
-	    xerror(errno, Ngt("cannot print to standard output"));
+	    xerror(errno, Ngt("cannot print to the standard output"));
 	free(q);
 	if (r < 0)
 	    return false;
@@ -1166,23 +1164,29 @@ bool print_trap(const wchar_t *signame, const wchar_t *command)
 }
 
 #if YASH_ENABLE_HELP
-const char trap_help[] = Ngt(
-"trap - set signal handler\n"
+const char *trap_help[] = { Ngt(
+"trap - set or print signal handlers\n"
+), Ngt(
 "\ttrap [action signal...]\n"
 "\ttrap signum [signal...]\n"
 "\ttrap -p [signal...]\n"
-"Sets the signal handler of the specified <signal>s to <action>.\n"
-"When the shell receives the signal, the corresponding action is executed as\n"
-"if by the \"eval\" command.\n"
+), Ngt(
+"The trap built-in sets the signal handler of the specified <signal>s to\n"
+"<action>. When the shell receives the signal, the corresponding action is\n"
+"executed as if by the eval built-in.\n"
+), Ngt(
 "If <action> is an empty string, no actions are taken and the signal is\n"
-"silently ignored when the signal is received.\n"
-"If <action> is \"-\", the signal handler is reset to the default.\n"
+"silently ignored when received.\n"
+"If <action> is `-', the signal handler is reset to the default.\n"
 "If the first operand is a non-negative integer, the operand is considered as\n"
-"a signal specification and <action> is assumed to be \"-\".\n"
+"a signal specification and <action> is assumed to be `-'.\n"
+), Ngt(
 "If the -p (--print) option is specified, the actions for the specified\n"
-"<signal>s are printed. This option is not available in POSIXly correct mode.\n"
+"<signal>s are printed. This option is not available in the POSIXly correct\n"
+"mode.\n"
+), Ngt(
 "Without any operands, all signal handlers currently set are printed.\n"
-);
+), NULL };
 #endif
 
 /* The "kill" builtin, which accepts the following options:
@@ -1192,59 +1196,78 @@ const char trap_help[] = Ngt(
  *  -v: prints signal info verbosely */
 int kill_builtin(int argc, void **argv)
 {
-#if YASH_ENABLE_HELP
-    if (!posixly_correct && argc == 2 && wcscmp(ARGV(1), L"--help") == 0)
-	return print_builtin_help(ARGV(0));
-#endif
-
-    wchar_t opt;
     int signum = SIGTERM;
     bool list = false, verbose = false;
 
-    xoptind = 0, xopterr = false;
-    while ((opt = xgetopt_long(argv,
-		    posixly_correct ? L"ls:" : L"+ln:s:v",
-		    NULL, NULL))) {
-	switch (opt) {
-	    /* we don't make any differences between -n and -s options */
-	    case L'n':  case L's':  parse_signal:
-		if (list)
-		    goto print_usage;
-		if (posixly_correct && wcsncmp(xoptarg, L"SIG", 3) == 0) {
-		    xerror(0, Ngt("%ls: signal name must be specified "
-				"without `SIG'"), xoptarg);
-		    return Exit_ERROR;
-		}
-		signum = get_signal_number_toupper(xoptarg);
-		if (signum < 0 || (signum == 0 && !iswdigit(xoptarg[0]))) {
-		    xerror(0, Ngt("%ls: no such signal"), xoptarg);
-		    return Exit_FAILURE;
-		}
-		goto no_more_options;
-	    case L'l':
-		list = true;
-		break;
-	    case L'v':
-		list = verbose = true;
-		break;
-	    default:
-		if (ARGV(xoptind)[0] == L'-' && ARGV(xoptind)[1] == xoptopt) {
+    /* We don't use the xgetopt function to parse options because the kill
+     * built-in has non-standard syntax. */
+    int optind;
+    for (optind = 1; optind < argc; optind++) {
+	wchar_t *arg = ARGV(optind);
+	if (arg[0] != L'-' || arg[1] == L'\0')
+	    break;
+	for (size_t i = 1; arg[i] != L'\0'; i++) {
+	    switch (arg[i]) {
+		case L'n':  case L's':
+		/* we don't make any differences between -n and -s options */
 		    if (list)
-			goto no_more_options;
-		    xoptarg = &ARGV(xoptind++)[1];
-		    goto parse_signal;
-		}
-		goto print_usage;
-	    no_more_options:
-		if (xoptind < argc && wcscmp(ARGV(xoptind), L"--") == 0)
-		    xoptind++;
-		goto main;
+			goto print_usage;
+		    arg = &arg[i + 1];
+		    if (*arg == L'\0') {
+			arg = ARGV(++optind);
+			if (arg == NULL) {
+			    xerror(0, Ngt("the signal name is not specified"));
+			    goto print_usage;
+			}
+		    }
+parse_signal_name:
+		    if (posixly_correct && matchwcsprefix(arg, L"SIG")) {
+			xerror(0, Ngt("%ls: the signal name must be specified "
+				    "without `SIG'"), arg);
+			return Exit_ERROR;
+		    }
+		    signum = get_signal_number_toupper(arg);
+		    if (signum < 0 || (signum == 0 && !iswdigit(*arg))) {
+			xerror(0, Ngt("no such signal `%ls'"), arg);
+			return Exit_FAILURE;
+		    }
+		    optind++;
+		    if (optind < argc && wcscmp(ARGV(optind), L"--") == 0)
+			optind++;
+		    goto main;
+		case L'l':
+		    list = true;
+		    break;
+		case L'v':
+		    list = verbose = true;
+		    break;
+		case L'-':
+		    if (i == 1) {
+			if (arg[2] == L'\0') {  /* `arg' is "--" */
+			    optind++;
+			    goto main;
+			}
+#if YASH_ENABLE_HELP
+			if (!posixly_correct && matchwcsprefix(L"--help", arg))
+			    return print_builtin_help(ARGV(0));
+#endif
+		    }
+		    /* falls thru! */
+		default:
+		    if (i == 1 && !list) {
+			arg = &arg[i];
+			goto parse_signal_name;
+		    } else {
+			xerror(0, Ngt("`%ls' is not a valid option"), arg);
+			goto print_usage;
+		    }
+	    }
 	}
     }
 
 main:
     if (list) {
-	if (xoptind == argc) {
+	if (optind == argc) {
 	    /* print info of all signals */
 	    for (const signal_T *s = signals; s->no; s++)
 		if (!print_signal(s->no, s->name, verbose))
@@ -1260,27 +1283,27 @@ main:
 		int signum;
 		const wchar_t *signame;
 
-		if (xwcstoi(ARGV(xoptind), 10, &signum) && signum >= 0) {
+		if (xwcstoi(ARGV(optind), 10, &signum) && signum >= 0) {
 		    if (signum >= TERMSIGOFFSET)
 			signum -= TERMSIGOFFSET;
 		    else if (signum >= (TERMSIGOFFSET & 0xFF))
 			signum -= (TERMSIGOFFSET & 0xFF);
 		} else {
-		    signum = get_signal_number_toupper(ARGV(xoptind));
+		    signum = get_signal_number_toupper(ARGV(optind));
 		}
 		signame = get_signal_name(signum);
 		if (signum <= 0 || signame[0] == L'\0')
-		    xerror(0, Ngt("%ls: no such signal"), ARGV(xoptind));
+		    xerror(0, Ngt("no such signal `%ls'"), ARGV(optind));
 		else if (!print_signal(signum, signame, verbose))
 		    return Exit_FAILURE;
-	    } while (++xoptind < argc);
+	    } while (++optind < argc);
 	}
     } else {
 	/* send signal */
-	if (xoptind == argc)
+	if (optind == argc)
 	    goto print_usage;
 	do {
-	    wchar_t *proc = ARGV(xoptind);
+	    wchar_t *proc = ARGV(optind);
 	    if (proc[0] == L'%') {
 		signal_job(signum, proc);
 	    } else {
@@ -1296,7 +1319,7 @@ main:
 		    continue;
 		}
 	    }
-	} while (++xoptind < argc);
+	} while (++optind < argc);
     }
     return (yash_error_message_count == 0) ? Exit_SUCCESS : Exit_FAILURE;
 
@@ -1334,7 +1357,7 @@ bool print_signal(int signum, const wchar_t *name, bool verbose)
     if (r >= 0) {
 	return true;
     } else {
-	xerror(errno, Ngt("cannot print to standard output"));
+	xerror(errno, Ngt("cannot print to the standard output"));
 	return false;
     }
 }
@@ -1352,21 +1375,24 @@ void signal_job(int signum, const wchar_t *jobspec)
 }
 
 #if YASH_ENABLE_HELP
-const char kill_help[] = Ngt(
+const char *kill_help[] = { Ngt(
 "kill - send a signal to processes\n"
+), Ngt(
 "\tkill [-signal|-s signal|-n number] process...\n"
 "\tkill -l [-v] [number...]\n"
+), Ngt(
 "The first form sends a signal to the specified processes. The signal to send\n"
 "can be specified by the name or by the number, which defaults to SIGTERM if\n"
 "not specified. The processes can be specified by the process ID or in the\n"
-"job specification format like \"%1\".\n"
-"If the process ID is negative, the signal is sent to the process group.\n"
+"job specification format like `%1'. If the process ID is negative, the\n"
+"signal is sent to the process group.\n"
+), Ngt(
 "The second form prints info about signals. For each <number> given, the name\n"
 "of the corresponding signal is printed. The <number> must be a valid signal\n"
 "number or the exit status of a process kill by a signal. If no <number>s are\n"
-"given, a list of available signals is printed.\n"
-"With the -v option, verbose info is printed.\n"
-);
+"given, a list of available signals is printed. With the -v option, the\n"
+"signal numbers and descriptions of signals are printed as well.\n"
+), NULL };
 #endif
 
 
