@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* strbuf.c: modifiable string buffer */
-/* (C) 2007-2010 magicant */
+/* (C) 2007-2011 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 
 #include "common.h"
 #include <assert.h>
+#include <errno.h>
 #include <stdarg.h>
 #include <stdbool.h>
 #include <stdio.h>
@@ -28,8 +29,17 @@
 #include "strbuf.h"
 #include "util.h"
 
+#if HAVE_WCSNRTOMBS && !defined(wcsnrtombs)
+size_t wcsnrtombs(char *restrict dst, const wchar_t **restrict src, size_t nwc,
+	size_t len, mbstate_t *restrict ps);
+#endif
+
+#ifndef XSTRBUF_INITSIZE
 #define XSTRBUF_INITSIZE 15
+#endif
+#ifndef XWCSBUF_INITSIZE
 #define XWCSBUF_INITSIZE 15
+#endif
 
 
 /* If the type of the return value of the functions below is string buffer,
@@ -41,7 +51,8 @@
 /* Initializes the specified string buffer as an empty string. */
 xstrbuf_T *sb_init(xstrbuf_T *buf)
 {
-    buf->contents = xmalloc((XSTRBUF_INITSIZE + 1) * sizeof (char));
+    // buf->contents = xmallocn(XSTRBUF_INITSIZE + 1, sizeof (char));
+    buf->contents = xmalloc(XSTRBUF_INITSIZE + 1);
     buf->contents[0] = '\0';
     buf->length = 0;
     buf->maxlength = XSTRBUF_INITSIZE;
@@ -59,12 +70,13 @@ xstrbuf_T *sb_initwith(xstrbuf_T *restrict buf, char *restrict s)
     return buf;
 }
 
-/* Changes the max length of the specified buffer.
+/* Changes the maximum length of the specified buffer.
  * If `newmax' is less than the current length of the buffer, the end of
  * the buffer contents is truncated. */
 xstrbuf_T *sb_setmax(xstrbuf_T *buf, size_t newmax)
 {
-    buf->contents = xrealloc(buf->contents, (newmax + 1) * sizeof (char));
+    // buf->contents = xreallocn(buf->contents, newmax + 1, sizeof (char));
+    buf->contents = xrealloc(buf->contents, newmax + 1);
     buf->maxlength = newmax;
     buf->contents[newmax] = '\0';
     if (newmax < buf->length)
@@ -147,9 +159,10 @@ xstrbuf_T *sb_ccat_repeat(xstrbuf_T *buf, char c, size_t n)
 
 /* Converts wide character `c' into a multibyte string and appends it to buffer
  * `buf'. Shift state `ps' is used for the conversion.
- * Returns true iff successful.
  * If `c' is a null character, the shift state is reset to the initial state but
- * the null character is not appended to the buffer. */
+ * the null character is not appended to the buffer.
+ * Returns true iff successful. On error, `errno' is set to EILSEQ and the state
+ * is left undefined. */
 bool sb_wccat(xstrbuf_T *restrict buf, wchar_t c, mbstate_t *restrict ps)
 {
     size_t count;
@@ -163,25 +176,73 @@ bool sb_wccat(xstrbuf_T *restrict buf, wchar_t c, mbstate_t *restrict ps)
     assert(0 < count && count <= buf->maxlength - buf->length);
     buf->length += count;
     if (c == L'\0')
-	buf->length--, assert(buf->contents[buf->length] == '\0');
+	buf->length--;
     else
 	buf->contents[buf->length] = '\0';
+    assert(buf->contents[buf->length] == '\0');
     return true;
 }
 
-/* Converts wide string `s' to a multibyte string and appends it to buffer
- * `buf'. Shift state `ps' is used for the conversion. After successful
- * conversion, `ps' will be the initial shift state.
+/* Appends first `n' characters of wide string `s' to multibyte buffer `buf'.
+ * The wide string is converted to multibyte string using shift state `ps'.
+ * If `n' is larger than the length of `s', the whole string is appended and
+ * the shift state is reset to the initial shift state.
+ * Returns NULL if the string is converted and appended successfully,
+ * otherwise a pointer to the character in `s' that caused the error.
+ * A partial result may be left in the buffer on error. */
+wchar_t *sb_wcsncat(xstrbuf_T *restrict buf,
+	const wchar_t *restrict s, size_t n, mbstate_t *restrict ps)
+{
+#if HAVE_WCSNRTOMBS
+    for (;;) {
+	const wchar_t *saves = s;
+	size_t count = wcsnrtombs(buf->contents + buf->length,
+		(const wchar_t **) &s, n,
+		buf->maxlength - buf->length, ps);
+	if (count == (size_t) -1) {
+	    buf->contents[buf->length] = '\0';
+	    break;
+	}
+	buf->length += count;
+	if (s == NULL)
+	    break;
+	assert((size_t) (s - saves) <= n);
+	n -= s - saves;
+	if (n == 0) {
+	    buf->contents[buf->length] = '\0';
+	    s = NULL;
+	    break;
+	}
+	sb_ensuremax(buf, buf->maxlength + MB_CUR_MAX);
+    }
+    assert(buf->contents[buf->length] == '\0');
+    return (wchar_t *) s;
+#else
+    while (n > 0) {
+	if (!sb_wccat(buf, *s, ps))
+	    return (wchar_t *) s;
+	if (*s == L'\0')
+	    return NULL;
+	s++, n--;
+    }
+    return NULL;
+#endif
+}
+
+/* Appends wide string `s' to multibyte buffer `buf'. The wide string is
+ * converted to multibyte string using shift state `ps'. After successful
+ * conversion, the shift state is reset to the initial shift state.
  * Returns NULL if the whole string is converted and appended successfully,
  * otherwise a pointer to the character in `s' that caused the error.
  * A partial result may be left in the buffer on error. */
+#if !HAVE_WCSNRTOMBS
 wchar_t *sb_wcscat(xstrbuf_T *restrict buf,
 	const wchar_t *restrict s, mbstate_t *restrict ps)
 {
     for (;;) {
 	size_t count = wcsrtombs(buf->contents + buf->length,
 		(const wchar_t **) &s,
-		buf->maxlength - buf->length + 1,
+		buf->maxlength - buf->length,
 		ps);
 	if (count == (size_t) -1) {
 	    buf->contents[buf->length] = '\0';
@@ -190,11 +251,12 @@ wchar_t *sb_wcscat(xstrbuf_T *restrict buf,
 	buf->length += count;
 	if (s == NULL)
 	    break;
-	sb_ensuremax(buf, buf->maxlength + 1);
+	sb_ensuremax(buf, buf->maxlength + MB_CUR_MAX);
     }
     assert(buf->contents[buf->length] == '\0');
     return (wchar_t *) s;
 }
+#endif
 
 /* Appends the result of `vsprintf' to the specified buffer.
  * `format' and the following arguments must not be part of `buf->contents'.
@@ -245,7 +307,7 @@ int sb_printf(xstrbuf_T *restrict buf, const char *restrict format, ...)
 /* Initializes the specified wide string buffer as an empty string. */
 xwcsbuf_T *wb_init(xwcsbuf_T *buf)
 {
-    buf->contents = xmalloc((XWCSBUF_INITSIZE + 1) * sizeof (wchar_t));
+    buf->contents = xmallocn(XWCSBUF_INITSIZE + 1, sizeof (wchar_t));
     buf->contents[0] = L'\0';
     buf->length = 0;
     buf->maxlength = XWCSBUF_INITSIZE;
@@ -263,12 +325,12 @@ xwcsbuf_T *wb_initwith(xwcsbuf_T *restrict buf, wchar_t *restrict s)
     return buf;
 }
 
-/* Changes the max length of the specified buffer.
+/* Changes the maximum length of the specified buffer.
  * If `newmax' is less than the current length of the buffer, the end of
  * the buffer contents is truncated. */
 xwcsbuf_T *wb_setmax(xwcsbuf_T *buf, size_t newmax)
 {
-    buf->contents = xrealloc(buf->contents, (newmax + 1) * sizeof (wchar_t));
+    buf->contents = xreallocn(buf->contents, newmax + 1, sizeof (wchar_t));
     buf->maxlength = newmax;
     buf->contents[newmax] = L'\0';
     if (newmax < buf->length)
@@ -338,17 +400,6 @@ xwcsbuf_T *wb_wccat(xwcsbuf_T *buf, wchar_t c)
     return buf;
 }
 
-/* Appends `n' characters of `c' to the end of buffer `buf'.
- * The characters are appended even if `c' is a null wide character. */
-xwcsbuf_T *wb_wccat_repeat(xwcsbuf_T *buf, wchar_t c, size_t n)
-{
-    wb_ensuremax(buf, buf->length + n);
-    wmemset(buf->contents + buf->length, c, n);
-    buf->length += n;
-    buf->contents[buf->length] = L'\0';
-    return buf;
-}
-
 /* Converts multibyte string `s' into a wide string and appends it to buffer
  * `buf'. The multibyte string is assumed to start in the initial shift state.
  * Returns NULL if the whole string is converted and appended successfully,
@@ -433,19 +484,23 @@ int wb_wprintf(xwcsbuf_T *restrict buf, const wchar_t *restrict format, ...)
  * The resulting string starts and ends in the initial shift state.*/
 char *malloc_wcsntombs(const wchar_t *s, size_t n)
 {
-    size_t nn = xwcsnlen(s, n);
-    if (s[nn] == L'\0')
-	return malloc_wcstombs(s);
+    xstrbuf_T buf;
+    mbstate_t state;
 
-    wchar_t ss[nn + 1];
-    wcsncpy(ss, s, nn);
-    ss[nn] = L'\0';
-    return malloc_wcstombs(ss);
+    sb_init(&buf);
+    memset(&state, 0, sizeof state);  // initialize as the initial shift state
+    if (sb_wcsncat(&buf, s, n, &state) == NULL) {
+	return sb_tostr(&buf);
+    } else {
+	sb_destroy(&buf);
+	return NULL;
+    }
 }
 
 /* Converts the specified wide string into a newly malloced multibyte string.
  * Returns NULL on error.
  * The resulting string starts and ends in the initial shift state.*/
+#if !HAVE_WCSNRTOMBS
 char *malloc_wcstombs(const wchar_t *s)
 {
     xstrbuf_T buf;
@@ -460,22 +515,7 @@ char *malloc_wcstombs(const wchar_t *s)
 	return NULL;
     }
 }
-
-/* Converts the specified multibyte string into a newly malloced wide string.
- * Only the first `n' bytes of `s' is converted at most.
- * Returns NULL on error.
- * The multibyte string is assumed to start in the initial shift state. */
-wchar_t *malloc_mbsntowcs(const char *s, size_t n)
-{
-    size_t nn = xstrnlen(s, n);
-    if (s[nn] == '\0')
-	return malloc_mbstowcs(s);
-
-    char ss[nn + 1];
-    strncpy(ss, s, nn);
-    ss[nn] = '\0';
-    return malloc_mbstowcs(ss);
-}
+#endif
 
 /* Converts the specified multibyte string into a newly malloced wide string.
  * Returns NULL on error. */
@@ -495,7 +535,19 @@ wchar_t *malloc_mbstowcs(const char *s)
 
 /********** Formatting Utilities **********/
 
-/* Returns the result of `sprintf' as a newly malloced string. */
+/* Returns the result of `vsprintf' as a newly malloced string.
+ * An error message is printed on error. The return value is non-NULL anyway. */
+char *malloc_vprintf(const char *format, va_list ap)
+{
+    xstrbuf_T buf;
+    sb_init(&buf);
+    if (sb_vprintf(&buf, format, ap) < 0)
+	xerror(errno, Ngt("unexpected error"));
+    return sb_tostr(&buf);
+}
+
+/* Returns the result of `sprintf' as a newly malloced string.
+ * An error message is printed on error. The return value is non-NULL anyway. */
 char *malloc_printf(const char *format, ...)
 {
     va_list ap;
@@ -506,7 +558,19 @@ char *malloc_printf(const char *format, ...)
     return result;
 }
 
-/* Returns the result of `swprintf' as a newly malloced string. */
+/* Returns the result of `vswprintf' as a newly malloced string.
+ * An error message is printed on error. The return value is non-NULL anyway. */
+wchar_t *malloc_vwprintf(const wchar_t *format, va_list ap)
+{
+    xwcsbuf_T buf;
+    wb_init(&buf);
+    if (wb_vwprintf(&buf, format, ap) < 0)
+	xerror(errno, Ngt("unexpected error"));
+    return wb_towcs(&buf);
+}
+
+/* Returns the result of `swprintf' as a newly malloced string.
+ * An error message is printed on error. The return value is non-NULL anyway. */
 wchar_t *malloc_wprintf(const wchar_t *format, ...)
 {
     va_list ap;
@@ -518,4 +582,39 @@ wchar_t *malloc_wprintf(const wchar_t *format, ...)
 }
 
 
+/********** String Creating Utilities **********/
+
+/* Joins the wide-character strings in the specified NULL-terminated array. The
+ * array elements are considered pointers to wide strings.
+ * If `padding' is non-NULL, `padding' is padded between each joined element.
+ * Returns a newly malloced string. */
+wchar_t *joinwcsarray(void *const *array, const wchar_t *padding)
+{
+    size_t elemcount, ccount = 0;
+
+    /* count the full length of the resulting string */
+    for (elemcount = 0; array[elemcount] != NULL; elemcount++)
+	ccount += wcslen(array[elemcount]);
+    if (padding != NULL && elemcount > 0)
+	ccount += wcslen(padding) * (elemcount - 1);
+
+    /* do copying */
+    wchar_t *const result = xmallocn(ccount + 1, sizeof *result);
+    wchar_t *s = result;
+    for (size_t i = 0; i < elemcount; i++) {
+	wchar_t *elem = array[i];
+	while (*elem != L'\0')
+	    *s++ = *elem++;
+	if (i + 1 < elemcount) {
+	    const wchar_t *pad = padding;
+	    while (*pad != L'\0')
+		*s++ = *pad++;
+	}
+    }
+    *s = L'\0';
+
+    return result;
+}
+
+
 /* vim: set ts=8 sts=4 sw=4 noet tw=80: */
