@@ -1,5 +1,5 @@
 # run-test.sh: runs tests specified by $TESTEE and $TEST_ITEMS
-# (C) 2007-2010 magicant
+# (C) 2007-2011 magicant
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -69,24 +69,19 @@ case "$1" in
 	;;
     array.y)
 	$INVOKE $TESTEE -c 'type array' 2>/dev/null | \
-	    grep '^array: regular builtin' >/dev/null
+	    grep '^array: a regular built-in' >/dev/null
 	;;
     dirstack.y)
 	$INVOKE $TESTEE -c 'type pushd' 2>/dev/null | \
-	    grep '^pushd: regular builtin' >/dev/null
+	    grep '^pushd: a regular built-in' >/dev/null
 	;;
     job.y)
 	# ensure that /dev/tty is available and that we're in foreground
-	{ <>/dev/tty; } 2>/dev/null &&
-	$INVOKE $TESTEE 2>/dev/null <<\END
-	trap 'kill $!; kill -s CONT 0' TTOU
-	$INVOKE $TESTEE -m -c ''&
-	wait
-END
+	./checkfg
 	;;
     help.y)
 	$INVOKE $TESTEE -c 'type help' 2>/dev/null | \
-	    grep '^help: regular builtin' >/dev/null
+	    grep '^help: a regular built-in' >/dev/null
 	;;
     history.y)
 	HISTFILE= $INVOKE $TESTEE -i +m --norcfile -c 'PATH=; fc -l' \
@@ -94,15 +89,15 @@ END
 	;;
     lineedit.y)
 	$INVOKE $TESTEE -c 'type bindkey' 2>/dev/null | \
-	    grep '^bindkey: regular builtin' >/dev/null
+	    grep '^bindkey: a regular built-in' >/dev/null
 	;;
     printf.y)
 	$INVOKE $TESTEE -c 'type printf' 2>/dev/null | \
-	    grep '^printf: regular builtin' >/dev/null
+	    grep '^printf: a regular built-in' >/dev/null
 	;;
     test.y)
 	$INVOKE $TESTEE -c 'type test' 2>/dev/null | \
-	    grep '^test: regular builtin' >/dev/null
+	    grep '^test: a regular built-in' >/dev/null
 	;;
 esac
 
