@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* yash.c: basic functions of the shell */
-/* (C) 2007-2010 magicant */
+/* (C) 2007-2011 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -57,7 +57,7 @@ extern int main(int argc, char **argv)
     __attribute__((nonnull));
 static struct input_file_info *new_input_file_info(int fd, size_t bufsize)
     __attribute__((malloc,warn_unused_result));
-static void execute_profile(void);
+static void execute_profile(const wchar_t *profile);
 static void execute_rcfile(const wchar_t *rcfile);
 static void print_help(void);
 static void print_version(void);
@@ -88,13 +88,7 @@ struct input_file_info *stdin_input_file_info;
 int main(int argc, char **argv)
 {
     void *wargv[argc + 1];
-    bool help = false, version = false;
-    bool do_job_control_set = false, is_interactive_set = false;
-    bool noprofile = false, norcfile = false;
-    bool option_error = false;
-    wchar_t opt;
     const wchar_t *shortest_name;
-    const wchar_t *rcfile = NULL;
 
     setvbuf(stdout, NULL, _IOLBF, BUFSIZ);
     setvbuf(stderr, NULL, _IOLBF, BUFSIZ);
@@ -110,9 +104,12 @@ int main(int argc, char **argv)
 	wargv[i] = malloc_mbstowcs(argv[i]);
 	if (wargv[i] == NULL) {
 	    fprintf(stderr,
-		    gt("%s: cannot convert the argument `%s' into wide-"
-			"character string: replaced with empty string\n"),
+		    gt("%s: cannot convert the argument `%s' "
+			"into a wide character string\n"),
 		    argv[0], argv[i]);
+	    fprintf(stderr,
+		    gt("%s: the argument is replaced with an empty string\n"),
+		    argv[0]);
 	    wargv[i] = xwcsdup(L"");
 	}
     }
@@ -134,99 +131,12 @@ int main(int argc, char **argv)
     if (wcscmp(shortest_name, L"sh") == 0)
 	posixly_correct = true;
 
-    /* parse options */
-    xoptind = 0;
-    xopterr = true;
-    while (!option_error &&
-	    (opt = xgetopt_long(wargv,
-		    L"+*cilo:sV" SHELLSET_OPTIONS,
-		    shell_long_options,
-		    &setoptindex))) {
-	switch (opt) {
-	case L'c':
-	    if (xoptopt != L'-') {
-		xerror(0, Ngt("%lc%lc: invalid option"),
-			(wint_t) xoptopt, (wint_t) L'c');
-		option_error = true;
-	    }
-	    shopt_read_arg = true;
-	    break;
-	case L'i':
-	    is_interactive = (xoptopt == L'-');
-	    is_interactive_set = true;
-	    break;
-	case L'l':
-	    is_login_shell = (xoptopt == L'-');
-	    break;
-	case L'L':
-	    set_option();
-	    break;
-	case L'o':
-	    do_job_control_set |= wcscmp(xoptarg, L"monitor") == 0;
-	    if (!set_long_option(xoptarg)) {
-		xerror(0, Ngt("%lco %ls: invalid option"),
-			(wint_t) xoptopt, xoptarg);
-		option_error = true;
-	    }
-	    break;
-	case L's':
-	    if (xoptopt != L'-') {
-		xerror(0, Ngt("%lc%lc: invalid option"),
-			(wint_t) xoptopt, (wint_t) L's');
-		option_error = true;
-	    }
-	    shopt_read_stdin = true;
-	    break;
-	case L'V':
-	    version = true;
-	    break;
-	case L'(':
-	    noprofile = true;
-	    break;
-	case L')':
-	    norcfile = true;
-	    break;
-	case L'!':
-	    rcfile = xoptarg;
-	    break;
-	case L'-':
-	    help = true;
-	    break;
-	case L'?':
-	    option_error = true;
-	    break;
-	case L'm':
-	    do_job_control_set = true;
-	    /* falls thru! */
-	default:
-	    set_single_option(opt);
-	    break;
-	}
-    }
-
-    if (option_error) {
-	print_help();
-	exit(Exit_ERROR);
-    }
-    if (version)
-	print_version();
-    if (help)
-	print_help();
-    if (version || help)
-	exit(Exit_SUCCESS);
-
-    /* ignore "-" if it's the first argument */
-    if (wargv[xoptind] && wcscmp(wargv[xoptind], L"-") == 0)
-	xoptind++;
-
-    /* `shell_pgid' must be initialized after the options have been parsed.
-     * This is required for the `set_monitor_option' function to work. */
     shell_pid = getpid();
     shell_pgid = getpgrp();
     stdin_input_file_info = new_input_file_info(STDIN_FILENO, 1);
     init_cmdhash();
     init_homedirhash();
-    init_variables();
+    init_environment();
     init_signal();
     init_shellfds();
     init_job();
@@ -235,20 +145,37 @@ int main(int argc, char **argv)
     init_alias();
 #endif
 
+    struct shell_invocation_T options = {
+	.profile = NULL, .rcfile = NULL,
+    };
+
+    int optresult = parse_shell_options(argc, wargv, &options);
+    if (optresult != Exit_SUCCESS)
+	exit(optresult);
+    if (options.version)
+	print_version();
+    if (options.help)
+	print_help();
+    if (options.version || options.help)
+	exit(yash_error_message_count == 0 ? Exit_SUCCESS : Exit_FAILURE);
+
+    init_variables();
+
     union {
 	wchar_t *command;
 	int fd;
     } input;
     const char *inputname;
 
-    if (shopt_read_arg && shopt_read_stdin) {
-	xerror(0, Ngt("both of -c and -s options cannot be used at once"));
+    if (shopt_cmdline && shopt_stdin) {
+	xerror(0, Ngt("the -c and -s options cannot be used both at once"));
 	exit(Exit_ERROR);
     }
-    if (shopt_read_arg) {
+    if (shopt_cmdline) {
 	input.command = wargv[xoptind++];
 	if (input.command == NULL) {
-	    xerror(0, Ngt("-c option specified but no command given"));
+	    xerror(0, Ngt("the -c option is specified "
+			"but no command is given"));
 	    exit(Exit_ERROR);
 	}
 	if (xoptind < argc) {
@@ -259,11 +186,11 @@ int main(int argc, char **argv)
 	}
     } else {
 	if (argc == xoptind)
-	    shopt_read_stdin = true;
-	if (shopt_read_stdin) {
+	    shopt_stdin = true;
+	if (shopt_stdin) {
 	    input.fd = STDIN_FILENO;
 	    inputname = NULL;
-	    if (!is_interactive_set && argc == xoptind
+	    if (!options.is_interactive_set && argc == xoptind
 		    && isatty(STDIN_FILENO) && isatty(STDERR_FILENO))
 		is_interactive = true;
 	    unset_nonblocking(STDIN_FILENO);
@@ -274,7 +201,7 @@ int main(int argc, char **argv)
 	    input.fd = move_to_shellfd(open(inputname, O_RDONLY));
 	    if (input.fd < 0) {
 		int errno_ = errno;
-		xerror(errno_, Ngt("cannot open `%s'"), inputname);
+		xerror(errno_, Ngt("cannot open file `%s'"), inputname);
 		exit(errno_ == ENOENT ? Exit_NOTFOUND : Exit_NOEXEC);
 	    }
 	}
@@ -282,14 +209,13 @@ int main(int argc, char **argv)
 
 #if YASH_ENABLE_LINEEDIT
     /* enable lineedit if interactive and connected to a terminal */
-    if (is_interactive && shopt_lineedit == shopt_nolineedit)
-	if (!is_interactive_set
-		|| (isatty(STDIN_FILENO) && isatty(STDERR_FILENO)))
-	    shopt_lineedit = shopt_vi;
+    if (!options.lineedit_set && shopt_lineedit == SHOPT_NOLINEEDIT)
+	if (is_interactive && isatty(STDIN_FILENO) && isatty(STDERR_FILENO))
+	    set_lineedit_option(SHOPT_VI);
 #endif
 
     is_interactive_now = is_interactive;
-    if (!do_job_control_set)
+    if (!options.do_job_control_set)
 	do_job_control = is_interactive;
     if (do_job_control) {
 	open_ttyfd();
@@ -299,14 +225,14 @@ int main(int argc, char **argv)
     set_signals();
     set_positional_parameters(wargv + xoptind);
 
-    if (is_login_shell && !posixly_correct && !noprofile)
+    if (is_login_shell && !posixly_correct && !options.noprofile)
 	if (getuid() == geteuid() && getgid() == getegid())
-	    execute_profile();
-    if (is_interactive && !norcfile)
+	    execute_profile(options.profile);
+    if (is_interactive && !options.norcfile)
 	if (getuid() == geteuid() && getgid() == getegid())
-	    execute_rcfile(rcfile);
+	    execute_rcfile(options.rcfile);
 
-    if (shopt_read_arg) {
+    if (shopt_cmdline) {
 	exec_wcs(input.command, inputname, true);
     } else {
 	exec_input(input.fd, inputname, is_interactive, true, true);
@@ -325,28 +251,35 @@ struct input_file_info *new_input_file_info(int fd, size_t bufsize)
 }
 
 /* Executes "$HOME/.yash_profile". */
-static void execute_profile(void)
+void execute_profile(const wchar_t *profile)
 {
-    wchar_t *wpath =
-	parse_and_expand_string(L"$HOME/.yash_profile", NULL, false);
-    if (wpath != NULL) {
-	char *path = realloc_wcstombs(wpath);
-	if (path != NULL) {
-	    int fd = move_to_shellfd(open(path, O_RDONLY));
-	    if (fd >= 0) {
-		exec_input(fd, path, false, true, false);
-		remove_shellfd(fd);
-		xclose(fd);
-	    }
-	    free(path);
-	}
+    char *path;
+
+    if (profile != NULL) {
+	path = malloc_wcstombs(profile);
+    } else {
+	wchar_t *wpath =
+	    parse_and_expand_string(L"$HOME/.yash_profile", NULL, false);
+	if (wpath == NULL)
+	    return;
+	path = realloc_wcstombs(wpath);
     }
+    if (path == NULL)
+	return;
+
+    int fd = move_to_shellfd(open(path, O_RDONLY));
+    if (fd >= 0) {
+	exec_input(fd, path, false, true, false);
+	remove_shellfd(fd);
+	xclose(fd);
+    }
+    free(path);
 }
 
 /* Executes the initialization file.
  * `rcfile' is the filename to source.
  * If `rcfile' is NULL, it defaults to "$HOME/.yashrc" or "$ENV". */
-static void execute_rcfile(const wchar_t *rcfile)
+void execute_rcfile(const wchar_t *rcfile)
 {
     char *path;
 
@@ -412,36 +345,35 @@ void exit_shell_with_status(int status)
 void print_help(void)
 {
     if (posixly_correct) {
-	printf(gt("Usage:  sh [options] [filename [args...]]\n"
-		  "        sh [options] -c command [command_name [args...]]\n"
-		  "        sh [options] -s [args...]\n"));
-	printf(gt("Options: -il%ls\n"), SHELLSET_OPTIONS);
+	xprintf(gt(
+	    "Usage:  sh [options] [filename [args...]]\n"
+	    "        sh [options] -c command [command_name [args...]]\n"
+	    "        sh [options] -s [args...]\n"));
+	xprintf(gt("Options: -abCefhilmnuvx\n"));
     } else {
-	printf(gt("Usage:  yash [options] [filename [args...]]\n"
-		  "        yash [options] -c command [command_name [args...]]\n"
-		  "        yash [options] -s [args...]\n"));
-	printf(gt("Options: -il%lsV\n"
-	          "         --interactive --login --noprofile "
-		  "--norcfile --rcfile=filename\n"),
-		SHELLSET_OPTIONS);
-	printf(gt("Type `set --help' in the shell for other options.\n"));
-//	printf(gt("Long options:\n"));
-//	for (size_t i = 0; shell_long_options[i].name; i++)
-//	    printf("\t--%ls\n", shell_long_options[i].name);
+	xprintf(gt(
+	    "Usage:  yash [options] [filename [args...]]\n"
+	    "        yash [options] -c command [command_name [args...]]\n"
+	    "        yash [options] -s [args...]\n"));
+	xprintf(gt(
+	    "Options: -abCefhilmnuVvx\n"
+	    "         --interactive --login --noprofile --norcfile \n"
+	    "         --profile=filename --rcfile=filename\n"));
+	xprintf(gt("Type `set --help' in the shell for other options.\n"));
     }
 }
 
 /* Prints the version info to the standard output. */
 void print_version(void)
 {
-    printf(gt("Yet another shell, version %s\n"), PACKAGE_VERSION);
-    printf(PACKAGE_COPYRIGHT "\n");
-    printf(gt("This is free software licensed under GNU GPL version 2.\n"
+    xprintf(gt("Yet another shell, version %s\n"), PACKAGE_VERSION);
+    xprintf(PACKAGE_COPYRIGHT "\n");
+    xprintf(gt("This is free software licensed under GNU GPL version 2.\n"
 	    "You can modify and redistribute it, but there is NO WARRANTY.\n"));
 
     if (shopt_verbose) {
-	printf(gt("\nCompiled %s %s\nEnabled features:\n"), __DATE__, __TIME__);
-	printf(""
+	xprintf(gt("\nEnabled features:\n"));
+	xprintf(""
 #ifndef NDEBUG
 		" * DEBUG\n"
 #endif
@@ -552,11 +484,7 @@ void exec_input(int fd, const char *name,
  * If no commands were executed, `laststatus' is set to zero. */
 void parse_and_exec(parseinfo_T *pinfo, bool finally_exit)
 {
-#ifdef NDEBUG
-    struct execinfo *ei;
-#else
-    struct execinfo *ei = ei;
-#endif
+    struct execinfo *INIT(ei);
     if (!finally_exit)
 	ei = save_execinfo();
 
@@ -567,7 +495,7 @@ void parse_and_exec(parseinfo_T *pinfo, bool finally_exit)
 	    forceexit = nextforceexit;
 	    nextforceexit = false;
 	    if (!is_interrupted() && return_pending()) {
-		xerror(0, Ngt("return: not in function or sourced file"));
+		xerror(0, Ngt("return: not in a function or sourced file"));
 		laststatus = Exit_FAILURE;
 	    }
 	    reset_execinfo();
@@ -581,7 +509,7 @@ void parse_and_exec(parseinfo_T *pinfo, bool finally_exit)
 	switch (read_and_parse(pinfo, &commands)) {
 	    case PR_OK:
 		if (commands) {
-		    if (!shopt_noexec) {
+		    if (shopt_exec || is_interactive) {
 			exec_and_or_lists(commands,
 				finally_exit && !pinfo->intrinput &&
 				pinfo->lastinputresult == INPUT_EOF);
@@ -632,26 +560,25 @@ bool input_is_interactive_terminal(const parseinfo_T *pinfo)
 }
 
 
-/********** Builtins **********/
+/********** Built-ins **********/
 
-static const struct xoption force_help_options[] = {
-    { L"force", OPTARG_NONE, L'f', },
+static const struct xgetopt_T force_help_options[] = {
+    { L'f', L"force", OPTARG_NONE, true,  NULL, },
 #if YASH_ENABLE_HELP
-    { L"help",  OPTARG_NONE, L'-', },
+    { L'-', L"help",  OPTARG_NONE, false, NULL, },
 #endif
-    { NULL, 0, 0, },
+    { L'\0', NULL, 0, false, NULL, },
 };
 
-/* The "exit" builtin.
+/* The "exit" built-in.
  * If the shell is interactive, there are stopped jobs and the -f flag is not
  * specified, then prints a warning message and does not exit. */
 int exit_builtin(int argc, void **argv)
 {
-    wchar_t opt;
-
-    xoptind = 0, xopterr = true;
-    while ((opt = xgetopt_long(argv, L"f", force_help_options, NULL))) {
-	switch (opt) {
+    const struct xgetopt_T *opt;
+    xoptind = 0;
+    while ((opt = xgetopt(argv, force_help_options, 0)) != NULL) {
+	switch (opt->shortopt) {
 	    case L'f':
 		forceexit = true;
 		break;
@@ -671,8 +598,7 @@ int exit_builtin(int argc, void **argv)
     size_t sjc;
     if (is_interactive_now && !forceexit && (sjc = stopped_job_count()) > 0) {
 	fprintf(stderr,
-		ngt("You have %zu stopped job(s)!",
-		    "You have a stopped job!",
+		ngt("You have a stopped job!",
 		    "You have %zu stopped jobs!",
 		    sjc),
 		sjc);
@@ -697,28 +623,31 @@ int exit_builtin(int argc, void **argv)
 }
 
 #if YASH_ENABLE_HELP
-const char exit_help[] = Ngt(
-"exit - exit shell\n"
+const char *exit_help[] = { Ngt(
+"exit - exit the shell\n"
+), Ngt(
 "\texit [-f] [n]\n"
-"Exits the shell with the exit status of <n>.\n"
+), Ngt(
+"The exit built-in makes the shell terminate with the exit status of <n>.\n"
 "If <n> is not specified, it defaults to the exit status of the last executed\n"
 "command. <n> should be between 0 and 255 inclusive.\n"
+), Ngt(
 "If the shell is interactive and you have any stopped jobs, the shell prints\n"
 "a warning message and does not exit. Use the -f (--force) option or use\n"
 "`exit' twice in a row to avoid the warning and really exit.\n"
-);
+), NULL };
 #endif
 
-/* The "suspend" builtin, which accepts the following options:
+/* The "suspend" built-in, which accepts the following options:
  *  -f: suspend even if it may cause a deadlock. */
 int suspend_builtin(int argc, void **argv)
 {
     bool force = false;
-    wchar_t opt;
 
-    xoptind = 0, xopterr = true;
-    while ((opt = xgetopt_long(argv, L"f", force_help_options, NULL))) {
-	switch (opt) {
+    const struct xgetopt_T *opt;
+    xoptind = 0;
+    while ((opt = xgetopt(argv, force_help_options, 0)) != NULL) {
+	switch (opt->shortopt) {
 	    case L'f':
 		force = true;
 		break;
@@ -734,7 +663,7 @@ int suspend_builtin(int argc, void **argv)
     if (argc != xoptind)
 	goto print_usage;
     if (!force && is_interactive_now && getsid(0) == shell_pgid) {
-	xerror(0, Ngt("refusing to suspend because of possible deadlock.\n"
+	xerror(0, Ngt("refusing to suspend because of a possible deadlock.\n"
 		    "Use the -f option to suspend anyway."));
 	return Exit_FAILURE;
     }
@@ -748,14 +677,17 @@ int suspend_builtin(int argc, void **argv)
 }
 
 #if YASH_ENABLE_HELP
-const char suspend_help[] = Ngt(
-"suspend - suspend shell\n"
+const char *suspend_help[] = { Ngt(
+"suspend - suspend the shell\n"
+), Ngt(
 "\tsuspend [-f]\n"
-"Suspends the shell until it receives SIGCONT.\n"
+), Ngt(
+"The suspend built-in suspends the shell until it receives SIGCONT.\n"
+), Ngt(
 "If the shell is interactive and is a session leader, this command refuses to\n"
 "suspend it in order to avoid a possible deadlock. You can use the -f option\n"
 "to force the shell to suspend anyway.\n"
-);
+), NULL };
 #endif
 
 
